--/**
-- * Copyright (c) 2007 E.D.Willink and others.
-- * All rights reserved.   This program and the accompanying materials
-- * are made available under the terms of the Eclipse Public License v1.0
-- * which accompanies this distribution, and is available at
-- * http://www.eclipse.org/legal/epl-v10.html
-- *
-- * Contributors:
-- *   E.D.Willink - Initial API and implementation
-- */
--
-- The QVTr Parser
--

%options escape=$
%options la=2
%options table=java
%options fp=QVTrLPGParser,prefix=TK_
%options error-maps
%options scopes
%options margin=4
%options noserialize
%options package=org.eclipse.gmt.umlx.eqvtr.cst.parser
--%options template=dtParserTemplateD.g
%options import_terminals=QVTrLexer.g
%options ast_type=CSTNode
%options programming_language=java
%options action=("*.java", "/.", "./")
%options ParseTable=lpg.lpgjavaruntime.ParseTable
%options include_directory=".;../../../../../ocl/internal/parser"
    
$Define

	-- Definition of macros used in the parser template
	--
	$prs_stream_class /.AbstractLPGParser./
	$action_class /.$file_prefix./

	-- package namespace of the LPG Runtime API
	$lpg_ns /.lpg.lpgjavaruntime./
    $LPGParsersym_class /.QVTrLPGParserSymbols./
    $null /../
    
$End

$KeyWords
	checkonly
	default_values
	domain
	enforce
	extends
	implementedby
	import
	key
	opposite
	overrides
	primitive
	query
	relation
	replace
	top
	transformation
	when
	where
$End

$Terminals
	PLUS_PLUS     ::= '++'
$End

$Globals
/.
import org.eclipse.gmt.umlx.eqvtr.cst.*;
import org.eclipse.ocl.internal.cst.CollectionTypeCS;
import org.eclipse.ocl.internal.parser.$lex_stream_class;
import org.eclipse.ocl.internal.parser.$prs_stream_class;

./
$End

$Rules
	goal -> topLevelCS
$End

$Include
	OCLLPGParser.g
$End

$Define
	$copyright_contributions /.
*   E.D.Willink - Extended API and implementation for QVTr./
$End

$Headers
	/.			
		protected IdentifierCS createIdentifierCS(int token) {
			IdentifierCS result = EqvtrCSTFactory.eINSTANCE.createIdentifierCS();
			result.setValue(getTokenText(token));
			setOffsets(result, getIToken(token), getIToken(token));
			return result;
		}

		protected String createIdentifierOrUnderscore(int token) {
			String text = getTokenText(token);
			if ("_".equals(text))
				return createUniqueIdentifier();
			else
				return text;
		}

		protected IdentifierCS createIdentifierOrUnderscoreCS(int token) {
			String text = getTokenText(token);
			if ("_".equals(text))
				return createUniqueIdentifierCS(token);
			else
				return createIdentifierCS(token);
		}
		
		@Override protected SimpleNameCS createSimpleNameCS(SimpleTypeEnum type, String value) {
			return super.createSimpleNameCS(type, "_".equals(value) ? createUniqueIdentifier() : value);
		}

		private int _uniqueNameCount = 0;

		protected String createUniqueIdentifier() {
			return "_unique" + _uniqueNameCount++;
		}

		protected IdentifierCS createUniqueIdentifierCS(int token) {
			IdentifierCS result = EqvtrCSTFactory.eINSTANCE.createIdentifierCS();
			result.setValue(createUniqueIdentifier());
			setOffsets(result, getIToken(token), getIToken(token));
			return result;
		}
	./
$End

$Rules
--<topLevel> ::= ('import' <filename> ';' )* <transformation>*
-- epsilon provided by OCL grammar
	topLevelCS_0_ ::= $empty
		/.$BeginJava
					TopLevelCS result = EqvtrCSTFactory.eINSTANCE.createTopLevelCS();
					$setResult(result);
		  $EndJava
		./
	topLevelCS_1_ ::= topLevelCS_0_ import filenameCS ';'
		/.$NewCase./
	topLevelCS_1_ ::= topLevelCS_1_ import filenameCS ';'
		/.$BeginJava
					TopLevelCS result = (TopLevelCS)$getSym(1);
					result.getImportClause().add((FilenameCS)$getSym(3));
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
	topLevelCS -> topLevelCS_1_
	topLevelCS ::= topLevelCS_0_ transformationCS
		/.$NewCase./
	topLevelCS ::= topLevelCS transformationCS
		/.$BeginJava
					TransformationCS transformationCS = (TransformationCS)$getSym(2);
					TopLevelCS result = (TopLevelCS)$getSym(1);
					result.getTransformation().add(transformationCS);
					setOffsets(result, result, transformationCS);
					$setResult(result);
		  $EndJava
		./
		
--<filename> ::= <identifier>
	filenameCS ::= STRING_LITERAL
		/.$BeginJava
					String quotedFileName = getTokenText($getToken(1));
					FilenameCS result = EqvtrCSTFactory.eINSTANCE.createFilenameCS();
					result.setValue(quotedFileName.substring(1, quotedFileName.length()-1));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(1)));
					$setResult(result);
		  $EndJava
		./
	
--<transformation> ::= 'transformation' <identifier>
--						'(' <modelDecl> (; <modelDecl>)* ')'
--						['extends' <identifier> (',' <identifier>)* ]
--						'{' <keyDecl>* ( <relation> | <query> )* }'
	transformationCS_0_ ::= transformation identifierCS '(' modelDeclCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					ModelDeclCS modelDeclCS = (ModelDeclCS)$getSym(4);
					TransformationCS result = EqvtrCSTFactory.eINSTANCE.createTransformationCS();
					result.setIdentifier(identifierCS);
					result.getModelDecl().add(modelDeclCS);
					setOffsets(result, getIToken($getToken(1)), modelDeclCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS_0_ ::= transformationCS_0_ ';' modelDeclCS
		/.$BeginJava
					ModelDeclCS modelDeclCS = (ModelDeclCS)$getSym(3);
					TransformationCS result = (TransformationCS)$getSym(1);
					result.getModelDecl().add(modelDeclCS);
					setOffsets(result, result, modelDeclCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS_1_ -> transformationCS_0_ ')'
	transformationCS_2_ -> transformationCS_1_
	transformationCS_2_ ::= transformationCS_1_ extends identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					TransformationCS result = (TransformationCS)$getSym(1);
					result.setExtends(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS_3_ -> transformationCS_2_ '{'
	transformationCS_3_ ::= transformationCS_3_ keyDeclCS
		/.$BeginJava
					KeyDeclCS keyDeclCS = (KeyDeclCS)$getSym(2);
					TransformationCS result = (TransformationCS)$getSym(1);
					result.getKeyDecl().add(keyDeclCS);
					setOffsets(result, result, keyDeclCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS_4_ -> transformationCS_3_
	transformationCS_4_ ::= transformationCS_4_ queryCS
		/.$BeginJava
					QueryCS queryCS =(QueryCS)$getSym(2);
					TransformationCS result = (TransformationCS)$getSym(1);
					result.getQuery().add(queryCS);
					setOffsets(result, result, queryCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS_4_ ::= transformationCS_4_ relationCS
		/.$BeginJava
					RelationCS relationCS = (RelationCS)$getSym(2);
					TransformationCS result = (TransformationCS)$getSym(1);
					result.getRelation().add(relationCS);
					setOffsets(result, result, relationCS);
					$setResult(result);
		  $EndJava
		./
	transformationCS -> transformationCS_4_ '}'
	
--<modelDecl> ::= <modelId> ':' <metaModelId> (, <metaModelId>)*
	modelDeclCS_0_ ::= modelIdCS ':'
		/.$BeginJava
					IdentifierCS modelIdCS = (IdentifierCS)$getSym(1);
					ModelDeclCS result = EqvtrCSTFactory.eINSTANCE.createModelDeclCS();
					result.setModelId(modelIdCS);
					setOffsets(result, modelIdCS, getIToken($getToken(2)));
					$setResult(result);
		  $EndJava
		./
	modelDeclCS_0_ -> modelDeclCS ','
	modelDeclCS ::= modelDeclCS_0_ metaModelIdCS
		/.$BeginJava
					IdentifierCS metaModelIdCS = (IdentifierCS)$getSym(2);
					ModelDeclCS result = (ModelDeclCS)$getSym(1);
					result.getMetaModelId().add(metaModelIdCS);
					setOffsets(result, result, metaModelIdCS);
					$setResult(result);
		  $EndJava
		./
	
--<modelId> ::= <identifier>
	modelIdCS -> identifierCS
	
--<metaModelId> ::= <identifier>
	metaModelIdCS -> identifierCS

--<keyDecl> ::= 'key' <classId> '{' <propertyId> (, <propertyId>)* '}' ';'
	keyDeclCS_0_ ::= key classIdCS '{'
		/.$BeginJava
					PathNameCS classIdCS = (PathNameCS)$getSym(2);
					KeyDeclCS result = EqvtrCSTFactory.eINSTANCE.createKeyDeclCS();
					result.setClassId(classIdCS);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(3)));
					$setResult(result);
		  $EndJava
		./
	keyDeclCS_0_ -> keyDeclCS_1_ ','
	keyDeclCS_1_ ::= keyDeclCS_0_ propertyIdCS
		/.$BeginJava
					IdentifiedCS propertyIdCS = (IdentifiedCS)$getSym(2);
					KeyDeclCS result = (KeyDeclCS)$getSym(1);
					result.getPropertyId().add(propertyIdCS);
					setOffsets(result, result, propertyIdCS);
					$setResult(result);
		  $EndJava
		./
	keyDeclCS -> keyDeclCS_1_ '}' ';'
	
--<classId> ::= <identifier>
	classIdCS -> pathNameCS

--<propertyId> ::= <identifier>
	propertyIdCS ::= identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(1);
					IdentifiedCS result = EqvtrCSTFactory.eINSTANCE.createIdentifiedCS();
					result.setIdentifier(identifierCS);
					setOffsets(result, identifierCS, identifierCS);
					$setResult(result);
		  $EndJava
		./
	propertyIdCS ::= opposite '(' pathNameCS '.' identifierCS ')'
		/.$BeginJava
					PathNameCS pathNameCS = (PathNameCS)$getSym(3);
					IdentifierCS identifierCS = (IdentifierCS)$getSym(5);
					OppositePropertyIdCS result = EqvtrCSTFactory.eINSTANCE.createOppositePropertyIdCS();
					result.setIdentifier(identifierCS);
					result.setOpposite(pathNameCS);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(6)));
					$setResult(result);
		  $EndJava
		./

--<relation> ::= ['top'] 'relation' <identifier> ['overrides' <identifier>]
--				'{'
--				<varDeclaration>*
--				(<domain> | <primitiveTypeDomain>)+
--				<when>? <where>?
--				'}'
--<when> ::= 'when' '{' <oclExpressionCS> '}'
--<where> ::= 'where' '{' <oclExpressionCS> '}'
			relationCS_0_ ::= relation identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					RelationCS result = EqvtrCSTFactory.eINSTANCE.createRelationCS();
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./
	relationCS_1_ -> relationCS_0_
	relationCS_1_ ::= top relationCS_0_
		/.$BeginJava
					RelationCS result = (RelationCS)$getSym(2);
					result.setTop(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./
	relationCS_2_ -> relationCS_1_
	relationCS_2_ ::= relationCS_1_ overrides identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					RelationCS result = (RelationCS)$getSym(1);
					result.setOverrides(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./
	relationCS_3_ -> relationCS_2_ '{'
	relationCS_3_ ::= relationCS_3_ varDeclarationCS
		/.$BeginJava
					VarDeclarationCS varDeclarationCS = (VarDeclarationCS)$getSym(2);
					RelationCS result = (RelationCS)$getSym(1);
					result.getVarDeclaration().add(varDeclarationCS);
					setOffsets(result, result, varDeclarationCS);
					$setResult(result);
		  $EndJava
		./
	relationCS_4_ ::= relationCS_3_ abstractDomainCS
		/.$NewCase ./
	relationCS_4_ ::= relationCS_4_ abstractDomainCS
		/.$BeginJava
					AbstractDomainCS domainCS = (AbstractDomainCS)$getSym(2);
					RelationCS result = (RelationCS)$getSym(1);
					result.getDomain().add(domainCS);
					setOffsets(result, result, domainCS);
					$setResult(result);
		  $EndJava
		./
	relationCS_5_ -> relationCS_4_
	relationCS_5_ ::= relationCS_4_ when '{' oclStatementCS '}'
		/.$BeginJava
					RelationCS result = (RelationCS)$getSym(1);
					EList oclStatementCS = (EList)$getSym(4);
					if (oclStatementCS != null)
						result.getWhen().addAll(oclStatementCS);
					setOffsets(result, result, getIToken($getToken(5)));
					$setResult(result);
		  $EndJava
		./
	relationCS_6_ -> relationCS_5_
	relationCS_6_ ::= relationCS_5_ where '{' oclStatementCS '}'
		/.$BeginJava
					RelationCS result = (RelationCS)$getSym(1);
					EList oclStatementCS = (EList)$getSym(4);
					if (oclStatementCS != null)
						result.getWhere().addAll(oclStatementCS);
					setOffsets(result, result, getIToken($getToken(5)));
					$setResult(result);
		  $EndJava
		./
	relationCS -> relationCS_6_ '}'
	abstractDomainCS -> domainCS
	abstractDomainCS -> primitiveTypeDomainCS

--<varDeclaration> ::= <identifier> (, <identifier>)* ':' <typeCS> ';'
	varDeclarationCS_0 ::= identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(1);
					VarDeclarationCS result = EqvtrCSTFactory.eINSTANCE.createVarDeclarationCS();
					result.getVarDeclarationId().add(identifierCS);
					setOffsets(result, identifierCS, identifierCS);
					$setResult(result);
		  $EndJava
		./
	varDeclarationCS_0 ::= varDeclarationCS_0 ',' identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					VarDeclarationCS result = (VarDeclarationCS)$getSym(1);
					result.getVarDeclarationId().add(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./		
	varDeclarationCS ::= varDeclarationCS_0 ':' typeCS ';'
		/.$BeginJava
					VarDeclarationCS result = (VarDeclarationCS)$getSym(1);
					result.setType((TypeCS)$getSym(3));
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
	
--<domain> ::= [<checkEnforceQualifier>]
--				'domain' <modelId> [ <identifier> ] ':' <typeCS>
--				'{' <propertyTemplate>* '}' [ '{' <oclExpressionCS> '}' ]
--				['implementedby' <OperationCallExpCS>]
--				';'	
--<checkEnforceQualifier> ::= 'checkonly' | 'enforce'
	domainCS_0_ ::= domain modelIdCS templateCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					TemplateCS templateCS = (TemplateCS)$getSym(3);
					DomainCS result = EqvtrCSTFactory.eINSTANCE.createDomainCS();
					result.setModelId(identifierCS);
					result.setTemplate(templateCS);
					setOffsets(result, getIToken($getToken(1)), templateCS);
					$setResult(result);
		  $EndJava
		./
	domainCS_1_ -> domainCS_0_
	domainCS_1_ ::= checkonly domainCS_0_
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(2);
					result.setCheckonly(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./
	domainCS_1_ ::= enforce domainCS_0_
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(2);
					result.setEnforce(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./
	domainCS_1_ ::= replace domainCS_0_
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(2);
					result.setReplace(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./
	domainCS_2_ ::= domainCS_1_ default_values '{' defaultValuesCS '}'
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(1);
					EList defaultValuesCS = (EList)$getSym(4);
					if (defaultValuesCS != null)
						result.getDefaultValue().addAll(defaultValuesCS);
					setOffsets(result, result, getIToken($getToken(5)));
					$setResult(result);
		  $EndJava
		./
	domainCS_3_ ::= domainCS_1_ '{' oclStatementCS '}'
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(1);
					EList oclStatementCS = (EList)$getSym(3);
					if (oclStatementCS != null)
						result.getGuardExpression().addAll(oclStatementCS);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
	domainCS_6_ -> domainCS_1_
	domainCS_6_ -> domainCS_2_
	domainCS_6_ -> domainCS_3_
	domainCS_6_ ::= domainCS_2_ '{' oclStatementCS '}'
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(1);
					EList oclStatementCS = (EList)$getSym(3);
					if (oclStatementCS != null)
						result.getGuardExpression().addAll(oclStatementCS);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
	domainCS_6_ ::= domainCS_3_ default_values '{' defaultValuesCS '}'
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(1);
					EList defaultValuesCS = (EList)$getSym(4);
					if (defaultValuesCS != null)
						result.getDefaultValue().addAll(defaultValuesCS);
					setOffsets(result, result, getIToken($getToken(5)));
					$setResult(result);
		  $EndJava
		./
	domainCS -> domainCS_6_
	domainCS -> domainCS_6_ ';'
	domainCS ::= domainCS_6_ implementedby operationCallExpCS ';'
		/.$BeginJava
					DomainCS result = (DomainCS)$getSym(1);
					result.setImplementedBy((OperationCallExpCS)$getSym(3));
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
		
--<primitiveTypeDomain> ::= 'primitive' 'domain' <identifier> ':' <typeCS>
--				';'
	primitiveTypeDomainCS ::= primitive domain identifierCS ':' typeCS ';'
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					TypeCS typeCS = (TypeCS)$getSym(5);
					PrimitiveTypeDomainCS result = EqvtrCSTFactory.eINSTANCE.createPrimitiveTypeDomainCS();
					result.setIdentifier(identifierCS);
					result.setType(typeCS);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(6)));
					$setResult(result);
		  $EndJava
		./
		
--<query> ::= 'query' <pathNameCS> '(' [<paramDeclaration> (','
--			<paramDeclaration>)*] ')' ':'
--			[<paramDeclaration> (','
--			<paramDeclaration>)*]
--			(
--			';' | '{' <oclExpressionCS> '}'
--			)
	queryCS_0_ ::= query pathNameCS '('
		/.$BeginJava
					QueryCS result = EqvtrCSTFactory.eINSTANCE.createQueryCS();
					result.setPathName((PathNameCS)$getSym(2));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(3)));
					$setResult(result);
		  $EndJava
		./
	queryCS_0_ -> queryCS_1_ ','
	queryCS_1_ ::= queryCS_0_ paramDeclarationCS
		/.$BeginJava
					ParamDeclarationCS paramDeclarationCS = (ParamDeclarationCS)$getSym(2);
					QueryCS result = (QueryCS)$getSym(1);
					result.getInputParamDeclaration().add(paramDeclarationCS);
					setOffsets(result, result, paramDeclarationCS);
					$setResult(result);
		  $EndJava
		./
	queryCS_2_ ::= queryCS_1_ ')' ':' typeCS
		/.$BeginJava
					TypeCS typeCS = (TypeCS)$getSym(4);
					QueryCS result = (QueryCS)$getSym(1);
					result.setType(typeCS);
					setOffsets(result, result, typeCS);
					$setResult(result);
		  $EndJava
		./
	queryCS -> queryCS_2_ ';'
	queryCS ::= queryCS_2_ '{' oclExpressionCS '}'
		/.$BeginJava
					QueryCS result = (QueryCS)$getSym(1);
					result.setOclExpression((OCLExpressionCS)$getSym(3));
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./
		
--<oclExpressionCS> ::= <propertyCallExpCS>
--					| <variableExpCS>
--					| <literalExpCS>
--					| <letExpCS>
--					| <ifExpCS>
--					| <template>
--					| '(' <oclExpressionCS> ')'
--					| (<oclExpressionCS> ';')*
	oclExpressionCS -> templateCS
	
	oclStatementCS_0_ ::= oclExpressionCS
		/.$BeginJava
					EList<OCLExpressionCS> result = new BasicEList<OCLExpressionCS>();
					result.add((OCLExpressionCS)$getSym(1));
//					setOffsets(result, $getSym(1), $getSym(1));
					$setResult(result);
		  $EndJava
		./
	oclStatementCS_0_ ::= oclStatementCS_0_ ';' oclExpressionCS
		/.$BeginJava
					EList result = (EList)$getSym(1);
					result.add((OCLExpressionCS)$getSym(3));
//					setOffsets(result, result, $getSym(3));
					$setResult(result);
		  $EndJava
		./
	oclStatementCS -> oclStatementCS_0_
	oclStatementCS -> oclStatementCS_0_ ';'
	
	defaultValueCS ::= identifierCS '=' oclExpressionCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(1);
					OCLExpressionCS oclExpressionCS = (OCLExpressionCS)$getSym(3);
					DefaultValueCS result = EqvtrCSTFactory.eINSTANCE.createDefaultValueCS();
					result.setIdentifier(identifierCS);
					result.setInitialiser(oclExpressionCS);
					setOffsets(result, identifierCS, oclExpressionCS);
					$setResult(result);
		  $EndJava
		./
	defaultValuesCS_0_ ::= defaultValueCS
		/.$BeginJava
					DefaultValueCS defaultValueCS = (DefaultValueCS)$getSym(1);
					EList<DefaultValueCS> result = new BasicEList<DefaultValueCS>();
					result.add(defaultValueCS);
//					setOffsets(result, defaultValueCS, defaultValueCS);
					$setResult(result);
		  $EndJava
		./
	defaultValuesCS_0_ ::= defaultValuesCS_0_ ';' defaultValueCS
		/.$BeginJava
					DefaultValueCS defaultValueCS = (DefaultValueCS)$getSym(3);
					EList result = (EList)$getSym(1);
					result.add(defaultValueCS);
//					setOffsets(result, result, defaultValueCS);
					$setResult(result);
		  $EndJava
		./
	defaultValuesCS -> defaultValuesCS_0_
	defaultValuesCS -> defaultValuesCS_0_ ';'

--<template> ::= <objectTemplate>
--				| <collectionTemplate>
	templateCS -> objectTemplateCS
	templateCS -> collectionTemplateCS
	
--objectTemplate ::= [<identifier>] ':' <typeCS> '{'
--					<propertyTemplate>* '}'
	objectTypeCS -> tupleTypeCS
	objectTypeCS -> primitiveTypeCS
	objectTypeCS -> pathNameCS
	objectTemplateCS_0_ ::= objectTypeCS
		/.$BeginJava
					TypeCS typeCS = (TypeCS)$getSym(1);
					ObjectTemplateCS result = EqvtrCSTFactory.eINSTANCE.createObjectTemplateCS();
					result.setType(typeCS);
					setOffsets(result, typeCS, typeCS);
					$setResult(result);
		  $EndJava
		./	
	objectTemplateCS_1_ ::= IDENTIFIER ':' objectTypeCS '{'
		/.$NewCase./
	objectTemplateCS_1_ ::= relationIdentifier ':' objectTypeCS '{'
		/.$NewCase./
	objectTemplateCS_1_ ::= self ':' objectTypeCS '{'
		/.$BeginJava
					IdentifierCS identifierCS = createIdentifierOrUnderscoreCS($getToken(1));
					TypeCS typeCS = (TypeCS)$getSym(3);
					ObjectTemplateCS result = EqvtrCSTFactory.eINSTANCE.createObjectTemplateCS();
					result.setType(typeCS);
					result.setIdentifier(identifierCS);
					setOffsets(result, identifierCS, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
	objectTemplateCS_1_ ::= ':' typeCS '{' 
		/.$BeginJava
					ObjectTemplateCS result = EqvtrCSTFactory.eINSTANCE.createObjectTemplateCS();
					result.setIdentifier(createUniqueIdentifierCS($getToken(1)));
					result.setType((TypeCS)$getSym(2));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(3)));
					$setResult(result);
		  $EndJava
		./	
	objectTemplateCS_2_ -> objectTemplateCS_1_
	objectTemplateCS_2_ -> objectTemplateCS_3_ ','
	objectTemplateCS_3_ ::= objectTemplateCS_2_ propertyTemplateCS 
		/.$BeginJava
					ObjectTemplateCS result = (ObjectTemplateCS)$getSym(1);
					PropertyTemplateCS propertyTemplateCS = (PropertyTemplateCS)$getSym(2);
					result.getPropertyTemplate().add(propertyTemplateCS);
					setOffsets(result, result, propertyTemplateCS);
					$setResult(result);
		  $EndJava
		./	
	objectTemplateCS -> objectTemplateCS_1_ '}' 
	objectTemplateCS -> objectTemplateCS_3_ '}' 


--<propertyTemplate> ::= <identifier> '=' <oclExpressionCS>
	propertyTemplateCS ::= propertyIdCS '=' oclExpressionCS
		/.$BeginJava
					IdentifiedCS propertyIdCS = (IdentifiedCS)$getSym(1);
					PropertyTemplateCS result = EqvtrCSTFactory.eINSTANCE.createPropertyTemplateCS();
					OCLExpressionCS oclExpressionCS = (OCLExpressionCS)$getSym(3);
					result.setPropertyId(propertyIdCS);
					result.setOclExpression(oclExpressionCS);
					setOffsets(result, propertyIdCS, oclExpressionCS);
					$setResult(result);
		  $EndJava
		./
	
--<collectionTemplate> ::= <identifier> ':' <collectionTypeIdentifierCS>
--					'(' <typeCS> ')'
--					'{'
--					<setComprehensionExpression>
--					| <memberSelectionExprCS>
--					| <oclExpressionCSList>
--					(',' <oclExpressionCSList>)*
--					'}'
--<memberSelectionExprCS> ::= (<identifier> | <objectTemplate> | '_')
--							'++' (<identifier> | '_')
	collectionTemplateCS_1_ ::= IDENTIFIER ':' collectionTypeCS '{'
		/.$NewCase./
	collectionTemplateCS_1_ ::= relationIdentifier ':' collectionTypeCS '{'
		/.$NewCase./
	collectionTemplateCS_1_ ::= self ':' collectionTypeCS '{'
		/.$BeginJava
					IdentifierCS identifierCS = createIdentifierOrUnderscoreCS($getToken(1));
					CollectionTypeCS collectionTypeCS = (CollectionTypeCS)$getSym(3);
					CollectionTemplateCS result = EqvtrCSTFactory.eINSTANCE.createCollectionTemplateCS();
					result.setType(collectionTypeCS);
					result.setIdentifier(identifierCS);
					setOffsets(result, identifierCS, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
	collectionTemplateCS_2_ -> collectionTemplateCS_1_
	collectionTemplateCS_2_ -> collectionTemplateCS_3_ ','
	collectionTemplateCS_3_ ::= collectionTemplateCS_2_ identifierCS
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					IdentifiedCS identifiedCS = EqvtrCSTFactory.eINSTANCE.createIdentifiedCS();
					identifiedCS.setIdentifier(identifierCS);
					setOffsets(identifiedCS, identifierCS, identifierCS);
					CollectionTemplateCS result = (CollectionTemplateCS)$getSym(1);
					result.getHeadIdentifier().add(identifiedCS);
					setOffsets(result, result, identifiedCS);
					$setResult(result);
		  $EndJava
		./	
	collectionTemplateCS_3_ ::= collectionTemplateCS_2_ objectTemplateCS
		/.$BeginJava
					ObjectTemplateCS objectTemplateCS = (ObjectTemplateCS)$getSym(2);
					CollectionTemplateCS result = (CollectionTemplateCS)$getSym(1);
					result.getHeadIdentifier().add(objectTemplateCS);
					setOffsets(result, result, objectTemplateCS);
					$setResult(result);
		  $EndJava
		./	
	collectionTemplateCS_4_ ::= collectionTemplateCS_3_ PLUS_PLUS identifierCS
		/.$BeginJava
					IdentifierCS tailIdentifier = (IdentifierCS)$getSym(3);
					CollectionTemplateCS result = (CollectionTemplateCS)$getSym(1);
					result.setTailIdentifier(tailIdentifier);
					setOffsets(result, result, tailIdentifier);
					$setResult(result);
		  $EndJava
		./
	collectionTemplateCS -> collectionTemplateCS_1_ '}'
	collectionTemplateCS -> collectionTemplateCS_4_ '}'

	paramDeclarationCS_0_ ::= ':' typeCS
		/.$BeginJava
					ParamDeclarationCS result = EqvtrCSTFactory.eINSTANCE.createParamDeclarationCS();
					TypeCS typeCS = (TypeCS)$getSym(2);
					result.setType(typeCS);
					setOffsets(result, getIToken($getToken(1)), typeCS);
					$setResult(result);
		  $EndJava
		./
	paramDeclarationCS -> paramDeclarationCS_0_
	paramDeclarationCS ::= identifierCS paramDeclarationCS_0_
		/.$BeginJava
					IdentifierCS identifierCS = (IdentifierCS)$getSym(1);
					ParamDeclarationCS result = (ParamDeclarationCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, identifierCS, result);
					$setResult(result);
		  $EndJava
		./

	relationIdentifier -> checkonly
	relationIdentifier -> default_values
	relationIdentifier -> domain
	relationIdentifier -> enforce
	relationIdentifier -> extends
	relationIdentifier -> implementedby
	relationIdentifier -> import
	relationIdentifier -> key
	relationIdentifier -> opposite
	relationIdentifier -> overrides
	relationIdentifier -> primitive
	relationIdentifier -> query
	relationIdentifier -> relation
	relationIdentifier -> replace
	relationIdentifier -> top
	relationIdentifier -> transformation
	relationIdentifier -> when
	relationIdentifier -> where
--	relationIdentifier -> '_'
	
	pathNameCS ::= relationIdentifier
		/.$BeginJava
					PathNameCS result = createPathNameCS();
					result.getSequenceOfNames().add(createIdentifierOrUnderscore($getToken(1)));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(1)));
					$setResult(result);
		  $EndJava
		./

	simpleNameCS ::= relationIdentifier
		/.$BeginJava
					SimpleNameCS result = createSimpleNameCS(
								SimpleTypeEnum.IDENTIFIER_LITERAL,
								createIdentifierOrUnderscore($getToken(1))
							);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(1)));
					$setResult(result);
		  $EndJava
		./
	
	identifierCS ::= IDENTIFIER
		/.$NewCase ./
	identifierCS ::= relationIdentifier
		/.$NewCase ./
	identifierCS ::= self
		/.$BeginJava
					IdentifierCS result = createIdentifierOrUnderscoreCS($getToken(1));
					$setResult(result);
		  $EndJava
		./

-- FIXME covered by OCL already
--	operationCallExpCS ::=  pathNameCS '::' simpleNameCS isMarkedPreCS '(' argumentsCSopt ')'
--		/.$BeginJava
--					RelationCallExpCS result = EqvtrCSTFactory.eINSTANCE.createRelationCallExpCS();
--					result.setPathNameCS((PathNameCS)$getSym(1));
--					result.setIsMarkedPreCS(createIsMarkedPreCS(false));
--					result.setSimpleNameCS((SimpleNameCS)$getSym(3));
--					result.getArguments().addAll(uncheckedCast((EList<OCLExpressionCS>)null, $getSym(5)));
--					setOffsets(result, (CSTNode)$getSym(1), getIToken($getToken(6)));
--					$setResult(result);
--		  $EndJava
--		./
$End
	