--/**
-- * Copyright (c) 2007 E.D.Willink and others.
-- * All rights reserved.   This program and the accompanying materials
-- * are made available under the terms of the Eclipse Public License v1.0
-- * which accompanies this distribution, and is available at
-- * http://www.eclipse.org/legal/epl-v10.html
-- *
-- * Contributors:
-- *   E.D.Willink - Initial API and implementation
-- */
--
-- The KM3 Parser
--

%options escape=$
%options la=2
%options table=java
%options fp=KM3LPGParser,prefix=TK_
%options error-maps
%options scopes
%options margin=4
%options noserialize
%options package=org.eclipse.gmt.umlx.registry.km3.parser
--%options template=dtParserTemplateD.g
%options import_terminals=KM3Lexer.g
%options ast_type=CSTNode
%options programming_language=java
%options action=("*.java", "/.", "./")
%options ParseTable=lpg.lpgjavaruntime.ParseTable
%options include_directory="."
    
$Define

	-- Definition of macros used in the parser template
	--
	$prs_stream_class /.AbstractLPGParser./
	$action_class /.$file_prefix./

	-- package namespace of the LPG Runtime API
	$lpg_ns /.lpg.lpgjavaruntime./
    $LPGParsersym_class /.KM3LPGParserSymbols./
    $null /../
	$lex_stream_class /.AbstractLexer./
	$action_class /.$file_prefix./

	$NewCase
	/. $Header
				case $rule_number:./

	-- From dtParserTemplateD.g
	------------------------------------------------------
	$Header
	/.
				//
				// Rule $rule_number:  $rule_text
				//./

	$BeginAction
	/. $Header
				case $rule_number: {./

	$EndAction
	/.		  break;
				}./

	$BeginJava
	/.$BeginAction
					$symbol_declarations./

	$EndJava /.$EndAction./

	$NoAction
	/. $Header
				case $rule_number:
					break;./

	$NullAction
	/. $Header
				case $rule_number:
					$setResult(null);
					break;./
	
	$EmptyListAction
	/. $Header
				case $rule_number:
					$setResult(new BasicEList());
					break;./
	
	$BeginActions
	/.
		@SuppressWarnings("unchecked")
		public void ruleAction(int ruleNumber)
		{
			switch (ruleNumber) {
			./

	$EndActions
	/.
				default:
					break;
			}
			return;
		}./

	$additional_interfaces /../
	$action_class /.$file_prefix./
	$setSym1 /.dtParser.setSym1./
	$setResult /.dtParser.setSym1./
	$getSym /.dtParser.getSym./
	$getToken /.dtParser.getToken./
	$getIToken /.getIToken./
	$getLeftSpan /.dtParser.getFirstToken./
	$getRightSpan /.dtParser.getLastToken./
	$prs_stream /.prsStream./
	
	-- modified to include throwing exceptions
	$parserCore
	/.
	public class $action_class extends $prs_stream_class implements RuleAction$additional_interfaces {
		protected static ParseTable prs = new $prs_type();
		private DeterministicParser dtParser;

		public $action_class() {
			super();
		}

		public $action_class($lex_stream_class lexStream) {
			super(lexStream);
		}
 
		@Override public String[] orderedTerminalSymbols() { return $sym_type.orderedTerminalSymbols; }
		public String getTokenKindName(int kind) { return $sym_type.orderedTerminalSymbols[kind]; }			
		public int getEOFTokenKind() { return $prs_type.EOFT_SYMBOL; }
		public $action_class getParseStream() { return this; }
			
		@Override public $ast_type parser(Monitor monitor, int error_repair_count) {
			ParseTable prsTable = new $prs_type();

			try {
				dtParser = new DeterministicParser(monitor, this, prsTable, this);
			}
			catch (NotDeterministicParseTableException e) {
				throw new RuntimeException("****Error: Regenerate $prs_type.java with -NOBACKTRACK option");
			}
			catch (BadParseSymFileException e) {
				throw new RuntimeException("****Error: Bad Parser Symbol File -- $sym_type.java. Regenerate $prs_type.java");
			}

			try {
				return ($ast_type) dtParser.parse();
			}
			catch (BadParseException e) {
				reset(e.error_token); // point to error token

				DiagnoseParser diagnoseParser = new DiagnoseParser(this, prsTable);
				diagnoseParser.diagnose(e.error_token);
			}

			return null;
		}		

	./
$End

$Notice
	/./**
 * <copyright>
 *
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   E.D.Willink - Initial API and implementation
 *
 * </copyright>
 */
	./
$End

$Globals
	/.
	import org.eclipse.gmt.umlx.registry.km3.cst.*;
	import org.eclipse.ocl.internal.cst.CSTNode;
	import org.eclipse.gmt.umlx.cst.parser.AbstractLPGParser;
	import org.eclipse.gmt.umlx.cst.parser.AbstractLexer;
	
	import $lpg_ns.BadParseException;
	import $lpg_ns.BadParseSymFileException;
	import $lpg_ns.DeterministicParser;
	import $lpg_ns.DiagnoseParser;
	import $lpg_ns.Monitor;
	import $lpg_ns.NotDeterministicParseTableException;
	import $lpg_ns.ParseTable;
	import $lpg_ns.RuleAction;
	./
$End

$KeyWords
	abstract
	attribute
	class
	container
	datatype
	enumeration
	extends
	literal
	oppositeOf
	ordered
	package
	reference
	unique
$End


$Identifier
	IDENTIFIER
$End

$Terminals
	
	QUOTED_NAME
	INTEGER_LITERAL
	
	MINUS    ::= '-'
	MULTIPLY ::= '*'

	LBRACE   ::= '{'
	RBRACE   ::= '}'
	LBRACKET ::= '['
	RBRACKET ::= ']'

	COMMA      ::= ','
	COLON      ::= ':'
	COLONCOLON ::= '::'
	SEMICOLON  ::= ';'
	DOT        ::= '.'

$End

$EOF
    EOF_TOKEN
$End

$ERROR
    ERROR_TOKEN
$End

$Start
    goal
$End

$Headers
	/.$parserCore

	./
$End

$Rules
	/.$BeginActions./
	
	goal -> MetaModelCS
	
	MetaModelCS_0_ ::= $empty
		/.$BeginJava
					MetaModelCS result = KM3CSTFactory.eINSTANCE.createMetaModelCS();
					$setResult(result);
		  $EndJava
		./
	MetaModelCS -> MetaModelCS_0_
	MetaModelCS ::= MetaModelCS PackageCS
		/.$BeginJava
					MetaModelCS result = (MetaModelCS)$getSym(1);
					PackageCS packageCS = (PackageCS)$getSym(2);
					result.getPackages().add(packageCS);
					setOffsets(result, result, packageCS);
					$setResult(result);
		  $EndJava
		./
		
	PackageCS_0_ ::= package IdentifierCS
		/.$BeginJava
					PackageCS result = KM3CSTFactory.eINSTANCE.createPackageCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./	
	PackageCS_1_ -> PackageCS_0_ '{'
	PackageCS_1_ ::= PackageCS_1_ ClassCS
		/.$BeginJava
					PackageCS result = (PackageCS)$getSym(1);
					ClassCS classCS = (ClassCS)$getSym(2);
					result.getClasses().add(classCS);
					setOffsets(result, result, classCS);
					$setResult(result);
		  $EndJava
		./	
	PackageCS_1_ ::= PackageCS_1_ DataTypeCS
		/.$BeginJava
					PackageCS result = (PackageCS)$getSym(1);
					DataTypeCS dataTypeCS = (DataTypeCS)$getSym(2);
					result.getDataTypes().add(dataTypeCS);
					setOffsets(result, result, dataTypeCS);
					$setResult(result);
		  $EndJava
		./	
	PackageCS_1_ ::= PackageCS_1_ EnumerationCS
		/.$BeginJava
					PackageCS result = (PackageCS)$getSym(1);
					EnumerationCS enumerationCS = (EnumerationCS)$getSym(2);
					result.getEnumerations().add(enumerationCS);
					setOffsets(result, result, enumerationCS);
					$setResult(result);
		  $EndJava
		./	
	PackageCS_1_ ::= PackageCS_1_ PackageCS
		/.$BeginJava
					PackageCS result = (PackageCS)$getSym(1);
					PackageCS packageCS = (PackageCS)$getSym(2);
					result.getPackages().add(packageCS);
					setOffsets(result, result, packageCS);
					$setResult(result);
		  $EndJava
		./	
	PackageCS -> PackageCS_1_ '}'
		
	ClassCS_0_ ::= class IdentifierCS
		/.$BeginJava
					ClassCS result = KM3CSTFactory.eINSTANCE.createClassCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./	
	ClassCS_1_ -> ClassCS_0_
	ClassCS_1_ ::=  abstract ClassCS_0_
		/.$BeginJava
					ClassCS result = (ClassCS)$getSym(2);
					result.setAbstract(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./	
	ClassCS_2_ -> ClassCS_1_ extends
	ClassCS_2_ -> ClassCS_3_ ','
	ClassCS_3_ ::= ClassCS_2_ IdentifiersCS
		/.$BeginJava
					ClassCS result = (ClassCS)$getSym(1);
					IdentifiersCS identifierCS = (IdentifiersCS)$getSym(2);
					result.getSupertypes().add(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./	
	ClassCS_4_ -> ClassCS_1_ '{'
	ClassCS_4_ -> ClassCS_3_ '{'
	ClassCS_4_ ::= ClassCS_4_ AttributeCS
		/.$BeginJava
					ClassCS result = (ClassCS)$getSym(1);
					AttributeCS attributeCS = (AttributeCS)$getSym(2);
					result.getAttributes().add(attributeCS);
					setOffsets(result, result, attributeCS);
					$setResult(result);
		  $EndJava
		./	
	ClassCS_4_ ::= ClassCS_4_ ReferenceCS
		/.$BeginJava
					ClassCS result = (ClassCS)$getSym(1);
					ReferenceCS referenceCS = (ReferenceCS)$getSym(2);
					result.getReferences().add(referenceCS);
					setOffsets(result, result, referenceCS);
					$setResult(result);
		  $EndJava
		./	
	ClassCS -> ClassCS_4_ '}'
		
	AttributeCS_0_ ::= attribute IdentifierCS
		/.$BeginJava
					AttributeCS result = KM3CSTFactory.eINSTANCE.createAttributeCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./	
	AttributeCS_1_ -> AttributeCS_0_
	AttributeCS_1_ ::=  unique AttributeCS_0_
		/.$BeginJava
					AttributeCS result = (AttributeCS)$getSym(2);
					result.setUnique(true);
					setOffsets(result, getIToken($getToken(1)), result);
					$setResult(result);
		  $EndJava
		./	
	AttributeCS_2_ -> AttributeCS_1_
	AttributeCS_2_ ::= AttributeCS_1_ '[' LowerMultiplicityBoundCS '-' UpperMultiplicityBoundCS ']'
		/.$BeginJava
					AttributeCS result = (AttributeCS)$getSym(1);
					MultiplicityBoundCS lowerBound = (MultiplicityBoundCS)$getSym(3);
					MultiplicityBoundCS upperBound = (MultiplicityBoundCS)$getSym(5);
					result.setLower(lowerBound);
					result.setUpper(upperBound);
					setOffsets(result, result, getIToken($getToken(6)));
					$setResult(result);
		  $EndJava
		./	
	AttributeCS_2_ ::= AttributeCS_1_ '[' StarMultiplicityBoundCS ']'
		/.$BeginJava
					AttributeCS result = (AttributeCS)$getSym(1);
					MultiplicityBoundCS upperBound = (MultiplicityBoundCS)$getSym(3);
					result.setUpper(upperBound);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
	AttributeCS_3_ -> AttributeCS_2_
	AttributeCS_3_ ::= AttributeCS_2_ ordered
		/.$BeginJava
					AttributeCS result = (AttributeCS)$getSym(1);
					result.setOrdered(true);
					setOffsets(result, result, getIToken($getToken(2)));
					$setResult(result);
		  $EndJava
		./	
	AttributeCS ::= AttributeCS_3_  ':' IdentifiersCS ';'
		/.$BeginJava
					AttributeCS result = (AttributeCS)$getSym(1);
					IdentifiersCS identifierCS = (IdentifiersCS)$getSym(3);
					result.setTyperef(identifierCS);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
		
		
	ReferenceCS_0_ ::= reference IdentifierCS
		/.$BeginJava
					ReferenceCS result = KM3CSTFactory.eINSTANCE.createReferenceCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS_1_ -> ReferenceCS_0_
	ReferenceCS_1_ ::= ReferenceCS_0_ '[' LowerMultiplicityBoundCS '-' UpperMultiplicityBoundCS ']'
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					MultiplicityBoundCS lowerBound = (MultiplicityBoundCS)$getSym(3);
					MultiplicityBoundCS upperBound = (MultiplicityBoundCS)$getSym(5);
					result.setLower(lowerBound);
					result.setUpper(upperBound);
					setOffsets(result, result, getIToken($getToken(6)));
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS_1_ ::= ReferenceCS_0_ '[' StarMultiplicityBoundCS ']'
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					MultiplicityBoundCS upperBound = (MultiplicityBoundCS)$getSym(3);
					result.setUpper(upperBound);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS_2_ -> ReferenceCS_1_
	ReferenceCS_2_ ::= ReferenceCS_1_ ordered
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					result.setOrdered(true);
					setOffsets(result, result, getIToken($getToken(2)));
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS_3_ -> ReferenceCS_2_
	ReferenceCS_3_ ::= ReferenceCS_2_ container
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					result.setContainer(true);
					setOffsets(result, result, getIToken($getToken(2)));
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS_4_ ::= ReferenceCS_3_  ':' IdentifiersCS
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					IdentifiersCS identifierCS = (IdentifiersCS)$getSym(3);
					result.setTyperef(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./	
	ReferenceCS -> ReferenceCS_4_ ';'
	ReferenceCS ::= ReferenceCS_4_ oppositeOf IdentifierCS ';'
		/.$BeginJava
					ReferenceCS result = (ReferenceCS)$getSym(1);
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					result.setOppositeOf(identifierCS);
					setOffsets(result, result, getIToken($getToken(4)));
					$setResult(result);
		  $EndJava
		./	
		
	LowerMultiplicityBoundCS ::= INTEGER_LITERAL
		/.$BeginJava
					MultiplicityBoundCS result = KM3CSTFactory.eINSTANCE.createMultiplicityBoundCS();
					result.setValue(getTokenText($getToken(1)));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(1)));
					$setResult(result);
		  $EndJava
		./	
	StarMultiplicityBoundCS ::= '*'
		/.$BeginJava
					MultiplicityBoundCS result = KM3CSTFactory.eINSTANCE.createMultiplicityBoundCS();
					result.setValue(getTokenText($getToken(1)));
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(1)));
					$setResult(result);
		  $EndJava
		./	
	UpperMultiplicityBoundCS -> LowerMultiplicityBoundCS
	UpperMultiplicityBoundCS -> StarMultiplicityBoundCS
				
	DataTypeCS_0_ ::= datatype IdentifierCS
		/.$BeginJava
					DataTypeCS result = KM3CSTFactory.eINSTANCE.createDataTypeCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), identifierCS);
					$setResult(result);
		  $EndJava
		./	
	DataTypeCS -> DataTypeCS_0_ ';'
	DataTypeCS ::= DataTypeCS_0_ instance IdentifierCS
		/.$BeginJava
					DataTypeCS result = (DataTypeCS)$getSym(1);
					IdentifierCS identifierCS = (IdentifierCS)$getSym(3);
					result.setInstance(identifierCS);
					setOffsets(result, result, identifierCS);
					$setResult(result);
		  $EndJava
		./	
	
	EnumerationCS_0 ::= enumeration IdentifierCS '{'
		/.$BeginJava
					EnumerationCS result = KM3CSTFactory.eINSTANCE.createEnumerationCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(3)));
					$setResult(result);
		  $EndJava
		./	
	EnumerationCS_1 -> EnumerationCS_0
	EnumerationCS_1 -> EnumerationCS_2
	EnumerationCS_2 ::= EnumerationCS_1 LiteralCS
		/.$BeginJava
					EnumerationCS result = (EnumerationCS)$getSym(1);
					LiteralCS literalCS = (LiteralCS)$getSym(2);
					result.getLiterals().add(literalCS);
					setOffsets(result, result, literalCS);
					$setResult(result);
		  $EndJava
		./	
	EnumerationCS -> EnumerationCS_2 '}'
	
	LiteralCS ::= literal IdentifierCS ';'
		/.$BeginJava
					LiteralCS result = KM3CSTFactory.eINSTANCE.createLiteralCS();
					IdentifierCS identifierCS = (IdentifierCS)$getSym(2);
					result.setIdentifier(identifierCS);
					setOffsets(result, getIToken($getToken(1)), getIToken($getToken(3)));
					$setResult(result);
		  $EndJava
		./	
		
	KeywordCS -> abstract
	KeywordCS -> attribute
	KeywordCS -> class
	KeywordCS -> container
	KeywordCS -> datatype
	KeywordCS -> enumeration
	KeywordCS -> extends
	KeywordCS -> literal
	KeywordCS -> oppositeOf
	KeywordCS -> ordered
	KeywordCS -> package
	KeywordCS -> reference
	KeywordCS -> unique
	IdentifierOrKeywordCS -> KeywordCS
	IdentifierOrKeywordCS -> IDENTIFIER
	IdentifierCS ::= IdentifierOrKeywordCS
		/.$BeginJava
					int token = $getToken(1);
					IdentifierCS result = KM3CSTFactory.eINSTANCE.createIdentifierCS();
					result.setValue(getTokenText(token));
					setOffsets(result, getIToken(token), getIToken(token));
					$setResult(result);
		  $EndJava
		./	
		
	IdentifierCS ::= QUOTED_NAME
		/.$BeginJava
					int token = $getToken(1);
					String text = getTokenText(token);
					IdentifierCS result = KM3CSTFactory.eINSTANCE.createIdentifierCS();
					result.setValue(text.substring(1, text.length()-1));
					setOffsets(result, getIToken(token), getIToken(token));
					$setResult(result);
		  $EndJava
		./	
	IdentifiersCS ::= IdentifierOrKeywordCS
		/.$BeginJava
					int token = $getToken(1);
					IdentifiersCS result = KM3CSTFactory.eINSTANCE.createIdentifiersCS();
					result.getValues().add(getTokenText(token));
					setOffsets(result, getIToken(token), getIToken(token));
					$setResult(result);
		  $EndJava
		./	
	IdentifiersCS ::= IdentifiersCS COLONCOLON IdentifierOrKeywordCS
		/.$BeginJava
					IdentifiersCS result = (IdentifiersCS)$getSym(1);
					int token = $getToken(3);
					result.getValues().add(getTokenText(token));
					setOffsets(result, result, getIToken(token));
					$setResult(result);
		  $EndJava
		./
		
$End

$Trailers
	/.
		$EndActions
	}
	./
$End
	