/*******************************************************************************
 * Copyright (c) 2008 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.common.utils;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * DebugCounters supports call count instrumentation by invocation of increment from
 * the instruimentation site and of print from the report site.
 */
public class DebugCounters
{
	private static List<String> keys = new ArrayList<String>();
	private static Map<String, Integer> counters = new HashMap<String, Integer>();
	private static int keyCount = 0;
	
	protected static synchronized void increment(String key) {
		Integer counter = counters.get(key);
		if (counter == null) {
			keys.add(key);
			counters.put(key, Integer.valueOf(1));
		}
		else
			counters.put(key, Integer.valueOf(counter.intValue() + 1));
	}
	
	protected static synchronized void print(PrintStream s, String title) {
		if (keys.size() > keyCount) {
			keyCount = keys.size();
			for (int i = 0; i < keyCount; i++) {
				s.print(i);
				s.append(" ");
				s.println(keys.get(i));
			}
			for (int i = 0; i < keyCount; i++) {
				s.format("%2d", Integer.valueOf(i));
				s.append(" ");
			}
			s.println("");
		}
		for (int i = 0; i < keyCount; i++) {
			s.format("%2d", counters.get(keys.get(i)));
			s.append(" ");
		}
		s.println(title);
		Integer zero = Integer.valueOf(0);
		for (String key : keys)
			counters.put(key, zero);
	}
}