/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.editor.actions;

import java.util.Iterator;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmt.umlx.editor.paged.PagedEditor;
import org.eclipse.gmt.umlx.editor.xml.EcoreXMLPageFactory;
import org.eclipse.gmt.umlx.editor.xml.XMLPageFactoryRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

public class OpenEcoreSheetAction extends AbstractObjectActionDelegate
{
	public void run(IAction action) {
		if (selection instanceof IStructuredSelection) {
			for (Iterator<?> it = ((IStructuredSelection) selection).iterator(); it.hasNext(); ) {
				Object element = it.next();
				XMLResource resource = null;
				if (element instanceof XMLResource)
					resource = (XMLResource) element;
				else if (element instanceof IAdaptable)
					resource = (XMLResource) ((IAdaptable) element).getAdapter(XMLResource.class);
				if ((resource != null) && (targetPart instanceof PagedEditor)) {
					PagedEditor pagedEditor = ((PagedEditor) targetPart);
					XMLPageFactoryRegistry pageFactoryRegistry = pagedEditor.getPageFactoryRegistry(XMLPageFactoryRegistry.class);
					if (pageFactoryRegistry != null)
						pageFactoryRegistry.createPageFor(pagedEditor, "Ecore", resource);
				}
			}
		}
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		super.selectionChanged(action, selection);
		if (!(selection instanceof IStructuredSelection)) {
			action.setEnabled(false);
			return;					
		}
		for (Iterator<?> it = ((IStructuredSelection) selection).iterator(); it.hasNext(); ) {
			Object element = it.next();
			if (!(element instanceof XMLResource)) {
				action.setEnabled(false);
				return;					
			}
			if (!EcoreXMLPageFactory.isEPackageBased((XMLResource) element)) {
				action.setEnabled(false);
				return;					
			}
		}
		action.setEnabled(true);
	}
}