/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.editor.common;

import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmt.umlx.editor.ICreationFactory;
import org.eclipse.gmt.umlx.qvt.ui.preferences.QVTPreferences;
import org.eclipse.qvt.declarative.ecore.utils.XMIUtils;

public abstract class AbstractCreationFactory implements ICreationFactory
{
	public void assignXmiIds(XMLResource resource) {
		XMIUtils.assignIds(resource, XMIUtils.uuidCreator, null);
	}

	@SuppressWarnings("unchecked")
	public <T> T getAdapter(Class<T> key) {
		if (key.isAssignableFrom(getClass()))
			return (T) this;
		return null;
	}
	
	public final String getEMOFExtension() {
		return QVTPreferences.getExtension(getEMOFExtensions());	
	}
	
	public final String getEcoreExtension() {
		return QVTPreferences.getExtension(getEcoreExtensions());	
	}
	
	public final String getXMLExtension() {
		boolean generateEMOF = QVTPreferences.generateEMOF();
		return generateEMOF ? getEMOFExtension() : getEcoreExtension();
	}
}
