/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.editor.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gmt.umlx.editor.text.TextColorManager;
import org.eclipse.gmt.umlx.editor.text.TextWhitespaceDetector;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.WhitespaceRule;

/**
 * Defines a scanner to process the residual NiceXSL partition.
 */
public class TextDefaultContentScanner extends RuleBasedScanner {
    public TextDefaultContentScanner(TextColorManager colorManager) {
    	List<IRule> rules = new ArrayList<IRule>();
        rules.add(new SingleLineRule("\"", "\"",
            colorManager.getDoubleQuotedStringToken()));
        rules.add(new SingleLineRule("'", "'",
            colorManager.getSingleQuotedStringToken()));
		rules.add(new EndOfLineRule("\"", 					// Nice idea
			colorManager.getUnterminatedStringToken()));	//  doesn't work
		rules.add(new EndOfLineRule("\'", 					// Nice idea
			colorManager.getUnterminatedStringToken()));	//  doesn't work
		rules.add(new EndOfLineRule("--",
			colorManager.getSingleLineCommentToken()));
		rules.add(new EndOfLineRule("//",
			colorManager.getSingleLineCommentToken()));
     	// Add generic whitespace rule.
    	rules.add(new WhitespaceRule(new TextWhitespaceDetector()));
    	IRule[] result = new IRule[rules.size()];
    	rules.toArray(result);
    	setRules(result);
    }
}
