/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.editor.text;

import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmt.umlx.editor.ICreationFactory;
import org.eclipse.gmt.umlx.editor.IMasterPageManager;
import org.eclipse.gmt.umlx.editor.paged.PagedEditor;
import org.eclipse.gmt.umlx.editor.pages.AbstractPageFactory;
import org.eclipse.gmt.umlx.editor.pages.PageDefinitions;

/**
 * A TextPageFactory manages creation of Textual CST edting pages.
 */
public class TextPageFactory extends AbstractPageFactory
{
	public TextPageFactory(PagedEditor pagedEditor, ICreationFactory creationFactory) {
		super(pagedEditor, creationFactory);
	}

	public IMasterPageManager createMasterPageFor(PagedEditor pagedEditor, XMLResource resource) {
		return createPageFor(pagedEditor, resource, true);
	}

	public void createOtherPagesFor(PagedEditor pagedEditor) {
		if (pagedEditor.getCreationFactory() != getCreationFactory())
			return;
		IMasterPageManager masterPageManager = pagedEditor.getMasterPageManager();
		if (!(masterPageManager instanceof TextPageManager))
			createPageFor(pagedEditor, masterPageManager.getEcoreResource(), false);
	}

	public TextPageManager createPageFor(PagedEditor pagedEditor, XMLResource resource, boolean isText) {
		TextPageManager pageManager = new TextPageManager(pagedEditor, resource, isText);
		pagedEditor.addPageManager(pageManager);
		return pageManager;
	}
	
	public void registerAsIn(String name, PageDefinitions pageFactoryRegistries) throws InstantiationException, IllegalAccessException {
		TextPageFactoryRegistry pageFactoryRegistry = pageFactoryRegistries.create(TextPageFactoryRegistry.class);
		pageFactoryRegistry.registerPageFactory(name, this);
	}
}