/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.editor.text;

import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmt.umlx.editor.IMasterPageManager;
import org.eclipse.gmt.umlx.editor.paged.PagedEditor;
import org.eclipse.gmt.umlx.editor.pages.AbstractPageFactoryRegistry;

/**
 * A TextPageFactoryRegistry manages creation of Text pages.
 */
public class TextPageFactoryRegistry extends AbstractPageFactoryRegistry<TextPageFactory>
{
	public IMasterPageManager createMasterPageFor(PagedEditor pagedEditor, XMLResource resource, boolean isText) {
		if (!isText)
			return null;
		for (TextPageFactory pageFactory : getPageFactories()) {
			IMasterPageManager masterPageManager = pageFactory.createMasterPageFor(pagedEditor, resource);
			if (masterPageManager != null)
				return masterPageManager;
		}
		return null;
	}

	public void createOtherPagesFor(PagedEditor pagedEditor) {
		for (TextPageFactory pageFactory : getPageFactories())
			pageFactory.createOtherPagesFor(pagedEditor);
	}

	public TextPageManager createPageFor(PagedEditor pagedEditor, XMLResource resource, boolean isText) {
		TextPageManager pageManager = new TextPageManager(pagedEditor, resource, isText);
		pagedEditor.addPageManager(pageManager);
		return pageManager;
	}
}
