package org.eclipse.gmt.umlx.eqvtc.editor;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.gmt.umlx.editor.IPageManager;
import org.eclipse.gmt.umlx.editor.common.AbstractCreationFactory;
import org.eclipse.gmt.umlx.eqvtc.cst.environment.QVTcTopLevelEnvironment;
import org.eclipse.gmt.umlx.eqvtc.cst.unparser.EqvtCoreUnparser;
import org.eclipse.gmt.umlx.qvt.ui.preferences.QVTPreferences;
import org.eclipse.ocl.lpg.ProblemHandler;
import org.eclipse.qvt.declarative.ecore.mappings.IMappingMetaData;
import org.eclipse.qvt.declarative.emof.QVTCore.facade.QVTCoreFacade;
import org.eclipse.qvt.declarative.registry.environment.AbstractFileHandle;

public class QVTcCreationFactory extends AbstractCreationFactory
{
	public static QVTcCreationFactory INSTANCE = new QVTcCreationFactory();
	public static final String BUILDER_ID = QVTcBuilder.class.getName();
	public static final String NATURE_ID = QVTcNature.class.getName();
	public static final String PROBLEM_MARKER_ID = QVTcMarkerProblemHandler.class.getPackage().getName() + ".qvtcProblem";
	
	protected QVTcCreationFactory() {}
	
	public ProblemHandler createProblemHandler(IFile file) {
		return new QVTcMarkerProblemHandler(file);
	}
	
	public QVTcTopLevelEnvironment createFileEnvironment(AbstractFileHandle fileHandle, ResourceSet resourceSet) {
		return new QVTcTopLevelEnvironment(fileHandle, resourceSet);
	}

	public QVTcTextEditor createTextEditor(IPageManager editorPageManager) {
		return new QVTcTextEditor(editorPageManager);
	}

	public EqvtCoreUnparser createUnparser(Resource resource) {
		return new EqvtCoreUnparser(resource);
	}

	public String getBuilderId() { return BUILDER_ID; }
	public String[] getEMOFExtensions() { return QVTPreferences.getCoreEMOFExtensions(); }
	public String[] getEcoreExtensions() { return QVTPreferences.getCoreEcoreExtensions(); }
	public String getEditorName() { return "QVT core"; }
	public IMappingMetaData getMappingMetaData() { return QVTCoreFacade.MAPPING_META_DATA; }	
	public String getNatureId() { return NATURE_ID; }
	public String getProblemMarkerId() { return PROBLEM_MARKER_ID; }
	public String getTextExtension() { return QVTPreferences.getCoreTextExtension(); }
	public String[] getTextExtensions() { return QVTPreferences.getCoreTextExtensions(); }
}
