/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.eqvtr.editor.common;

import java.text.MessageFormat;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.Resource.Factory;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.gmt.umlx.eqvtr.editor.text.QVTTextEditor;
import org.eclipse.gmt.umlx.eqvtr.editor.xml.XMLEditor;
import org.eclipse.gmt.umlx.qvt.ui.preferences.QVTPreferences;
import org.eclipse.gmt.umlx.registry.resource.EcoreFamilyResourceSetImpl;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.StatusTextEditor;
import org.xml.sax.SAXParseException;

public abstract class CopyOfQVTEditor extends CopyOfMultiPageTextEditor
{
	private QVTTextEditor textEditor;
	private XMLEditor xmlEditor;
	private MultiPageEditorPart ecoreEditor;
	private MultiPageEditorPart emofEditor;
	private int textEditorIndex = -1; 
	private int xmlEditorIndex = -1; 
	private int ecoreEditorIndex = -1; 
	private int emofEditorIndex = -1; 
	private StatusTextEditor masterEditor = null;
	private int currentPageIndex = -1;		// Workaround for Bug 177568
	/**
	 * Creates a multi-page editor example.
	 */
	public CopyOfQVTEditor() {
		super();
	}

	protected MultiPageEditorPart createEMOFEditor() { return null; }
	
	/**
	 * Creates the abstract syntax (XML) editor page of the multi-page editor.
	 */
	void createEMOFEditorPage() {
		emofEditor = createEMOFEditor();
		if (emofEditor == null)
			return;
		try {
			IFileEditorInput editorInput = (IFileEditorInput) getEditorInput();
//			if (!isMaster) {
				IFile textFile = editorInput.getFile();
				IPath concretePath = textFile.getProjectRelativePath();
				IPath abstractPath = concretePath.addFileExtension(getEMOFExtension());
				IFile xmlFile = textFile.getProject().getFile(abstractPath);
				editorInput = new DummyFileEditorInput(xmlFile);
//			}
			emofEditorIndex = addPage(emofEditor, editorInput);
			setPageText(emofEditorIndex, "EMOF");
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating EMOF editor",
				null,
				e.getStatus());
		}
	}

	protected MultiPageEditorPart createEcoreEditor() { return null; }
	
	/**
	 * Creates the abstract syntax (XML) editor page of the multi-page editor.
	 */
	void createEcoreEditorPage() {	
		ecoreEditor = createEcoreEditor();
		if (ecoreEditor == null)
			return;
		try {
			IFileEditorInput editorInput = (IFileEditorInput) getEditorInput();
//			if (!isMaster) {
				IFile textFile = editorInput.getFile();
				IPath concretePath = textFile.getProjectRelativePath();
				IPath abstractPath = concretePath.addFileExtension(getEcoreExtension());
				IFile xmlFile = textFile.getProject().getFile(abstractPath);
				editorInput = new DummyFileEditorInput(xmlFile);
//			}
			ecoreEditorIndex = addPage(ecoreEditor, editorInput);
			setPageText(ecoreEditorIndex, "Ecore");
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating Ecore editor",
				null,
				e.getStatus());
		}
	}

	@Override public void createModelGen() {
		// TODO Auto-generated method stub
		super.createModelGen();
	}

	/**
	 * Creates the pages of the multi-page editor.
	 */
	@Override public void createPages() {
		super.createPages();
		IFileEditorInput fileInput = (IFileEditorInput) getEditorInput();
		IFile iFile = fileInput.getFile();
		URI uri = URI.createPlatformResourceURI(iFile.getFullPath().toString(), true);
		ResourceSet resourceSet = new EcoreFamilyResourceSetImpl();
		Factory.Registry resourceFactoryRegistry = resourceSet.getResourceFactoryRegistry();
		resourceFactoryRegistry.getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
		boolean isXML = true;
		try {
			@SuppressWarnings("unused") Resource resource = resourceSet.getResource(uri, true);
		} catch (WrappedException e) {
			Exception ex = e.exception();
			if (ex instanceof SAXParseException)
				isXML = false;									// Resource analysis failed
		} catch(RuntimeException e) { 
			e.printStackTrace();
		}
		createTextEditorPage(!isXML);
		createXMLEditorPage(isXML);
//		createEcoreEditorPage();
//		createEMOFEditorPage();
		if (!isXML)
			masterEditor = textEditor;
		else
			masterEditor = xmlEditor;
		setActivePage(isXML ? xmlEditorIndex : textEditorIndex);
	}

	/**
	 * Create the concrete syntax (text) content from the abstract syntax (XML) content.
	 */
	protected IFileEditorInput createTextContent(IFile textFile) {
		IFileEditorInput xmlInput = (IFileEditorInput) xmlEditor.getEditorInput();
		IFile xmlFile = xmlInput.getFile();
		String fileName = xmlFile.getFullPath().toString();
		URI uri = URI.createPlatformResourceURI(fileName, true);
		IFile newTextFile = createTextFile(textFile, uri);
		return newTextFile != null ? new FileEditorInput(textFile) : null;
	}

	protected abstract QVTTextEditor createTextEditor();

	protected abstract IFile createTextFile(IFile textFile, URI uri);
	
	/**
	 * Creates the concrete syntax (text) editor page of the multi-page editor.
	 */
	void createTextEditorPage(boolean isMaster) {
		try {
			textEditor = createTextEditor();
			IFileEditorInput editorInput = (IFileEditorInput) getEditorInput();
			if (!isMaster) {
				IFile xmlFile = editorInput.getFile();
				IPath abstractPath = xmlFile.getProjectRelativePath();
				IPath concretePath = abstractPath.addFileExtension(getTextExtension());
				IFile textFile = xmlFile.getProject().getFile(concretePath);
				editorInput = new DummyFileEditorInput(textFile);
			}
			textEditorIndex = addPage(textEditor, editorInput);
			setPageText(textEditorIndex, "Text");
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating nested text editor",
				null,
				e.getStatus());
		}
	}
	
	/**
	 * Create abstract syntax (XML) content from the the concrete syntax (text) content. 
	 */
	IFileEditorInput createXMLContent(IFile xmlFile, boolean generateEMOF) {
		IFileEditorInput textInput = (IFileEditorInput) textEditor.getEditorInput();
		IFile textFile = textInput.getFile();
		String fileName = xmlFile.getFullPath().toString();
    	URI uri = URI.createPlatformResourceURI(fileName, true);
		createXMLFile(generateEMOF, textFile, uri);
		return new FileEditorInput(xmlFile);
	}
	
	/**
	 * Creates the abstract syntax (XML) editor page of the multi-page editor.
	 */
	void createXMLEditorPage(boolean isMaster) {
		try {
			IFileEditorInput editorInput = (IFileEditorInput) getEditorInput();
			if (!isMaster) {
				IFile textFile = editorInput.getFile();
				IPath concretePath = textFile.getProjectRelativePath();
				IPath abstractPath = concretePath.addFileExtension(getXMLExtension());
				IFile xmlFile = textFile.getProject().getFile(abstractPath);
				editorInput = new DummyFileEditorInput(xmlFile);
			}
			xmlEditor = new XMLEditor();
			xmlEditorIndex = addPage(xmlEditor, editorInput);
			setPageText(xmlEditorIndex, "XML");
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating nested XML editor",
				null,
				e.getStatus());
		}
	}

	protected abstract void createXMLFile(boolean generateEMOF, IFile textFile, URI uri);	
	
	@Override public void doSave(IProgressMonitor monitor) {
		super.doSave(monitor);
		IFileEditorInput textInput = (IFileEditorInput) textEditor.getEditorInput();
		refreshMarkers(textInput.getFile());
	}

	/**
	 * Saves the multi-page editor's document as another file.
	 * Also updates the text for page 0's tab, and updates this multi-page editor's input
	 * to correspond to the nested editor's.
	 */
	@Override public void doSaveAs() {
		AbstractTextEditor editor = getMasterEditor();
		final IEditorInput input = editor.getEditorInput();
		QVTSaveAsDialog dialog = new QVTSaveAsDialog(getSite().getShell())
	    {
			@Override protected String[] getSaveTypes() {
				return getSaveAsExtensionDescriptions();
			}
	    };
		IFile original = (input instanceof IFileEditorInput) ? ((IFileEditorInput) input).getFile() : null;
		if (original != null)
			dialog.setOriginalFile(original);
		dialog.create();
		if (dialog.open() == Window.CANCEL)
			return;
		IPath filePath = dialog.getResult();
		if (filePath == null)
			return;
    	IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IFile file = workspace.getRoot().getFile(filePath);
    	switch (dialog.getSaveTypeIndex()) {
	    	case 0: {
	    		doSaveAsText(file, getProgressMonitor());
	    		break;
	    	}
	    	case 1: {
	    		doSaveAsXML(file, getProgressMonitor(), false);
				break;
			}
	    	case 2: {
	    		doSaveAsXML(file, getProgressMonitor(), true);
				break;
			}
    	}
//		setPageText(0, editor.getTitle());		// FIXME carry on as if opening save as file
//		setInput(editor.getEditorInput());
	}
	
	protected void doSaveAsText(IFile file, IProgressMonitor progressMonitor) {
		if (textEditor == masterEditor) {
			IEditorInput input = textEditor.getEditorInput();
			IDocumentProvider provider = textEditor.getDocumentProvider();
			if (provider == null)
				return;	// editor has programmatically been  closed while the dialog was open
			final IEditorInput newInput = new FileEditorInput(file);
	//		boolean success= false;
			try {
	//			provider.aboutToChange(newInput);
				provider.saveDocument(progressMonitor, newInput, provider.getDocument(input), true);
	//			success = true;
	
			} catch (CoreException x) {
				final IStatus status= x.getStatus();
				if (status == null || status.getSeverity() != IStatus.CANCEL) {
					String title = EditorMessages.AbstractDecoratedTextEditor_error_saveAs_title;
					String msg = MessageFormat.format(EditorMessages.AbstractDecoratedTextEditor_error_saveAs_message, x.getMessage());
					MessageDialog.openError(getSite().getShell(), title, msg);
				}
			} finally {
	//			provider.changed(newInput);
	//			if (success)
	//				setInput(newInput);
			}
		}
		else
			createTextContent(file);
	}

	protected void doSaveAsXML(IFile file, IProgressMonitor progressMonitor, boolean generateEMOF) {
		boolean masterIsEMOF = QVTPreferences.generateEMOF();
		if ((xmlEditor == masterEditor) && (masterIsEMOF == generateEMOF)) {
			IEditorInput input = xmlEditor.getEditorInput();
			IDocumentProvider provider = xmlEditor.getDocumentProvider();
			if (provider == null)
				return;	// editor has programmatically been  closed while the dialog was open
			final IEditorInput newInput = new FileEditorInput(file);
//			boolean success= false;
			try {
//				provider.aboutToChange(newInput);
				provider.saveDocument(progressMonitor, newInput, provider.getDocument(input), true);
//				success = true;

			} catch (CoreException x) {
				final IStatus status= x.getStatus();
				if (status == null || status.getSeverity() != IStatus.CANCEL) {
					String title = EditorMessages.AbstractDecoratedTextEditor_error_saveAs_title;
					String msg = MessageFormat.format(EditorMessages.AbstractDecoratedTextEditor_error_saveAs_message, x.getMessage());
					MessageDialog.openError(getSite().getShell(), title, msg);
				}
			} finally {
//				provider.changed(newInput);
//				if (success)
//					setInput(newInput);
			}
		}
		else
			createXMLContent(file, generateEMOF);
	}

	private IEditorInput getEMOFEditorInput() {
		IFileEditorInput emofInput = (IFileEditorInput) emofEditor.getEditorInput();
		return (IEditorPart)masterEditor != (IEditorPart)emofEditor ? createXMLContent(emofInput.getFile(), true) : emofInput;
	}

	protected String getEMOFExtension() { return "xyzzy"; } // FIXME

	private IEditorInput getEcoreEditorInput() {
		IFileEditorInput ecoreInput = (IFileEditorInput) ecoreEditor.getEditorInput();
		return (IEditorPart)masterEditor != (IEditorPart)ecoreEditor ? createXMLContent(ecoreInput.getFile(), false) : ecoreInput;
	}

	protected String getEcoreExtension() { return "xyzzy"; } // FIXME

	@Override protected AbstractTextEditor getMasterEditor() {
		return masterEditor;
	}
	
	protected abstract String[] getSaveAsExtensionDescriptions();

	protected IEditorInput getTextEditorInput() {
		IFileEditorInput textInput = (IFileEditorInput) textEditor.getEditorInput();
		return masterEditor != textEditor ? createTextContent(textInput.getFile()) : textInput;
	}

	protected abstract String getTextExtension();

	protected IEditorInput getXMLEditorInput() {
		IFileEditorInput xmlInput = (IFileEditorInput) xmlEditor.getEditorInput();
		return masterEditor != xmlEditor ? createXMLContent(xmlInput.getFile(), QVTPreferences.generateEMOF()) : xmlInput;
	}

	protected abstract String getXMLExtension();
	
	/* (non-Javadoc)
	 * Method declared on IEditorPart
	 */
	@Override public void gotoMarker(IMarker marker) {
		setActivePage(0);
		IDE.gotoMarker(getEditor(0), marker);
	}
	
	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	@Override public void init(IEditorSite site, IEditorInput editorInput)
		/*throws PartInitException*/ {
//		if (!(editorInput instanceof IFileEditorInput))
//			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);
	}
	
	/**
	 * Calculates the contents of a page when it is activated.
	 */
	@Override protected void pageChange(int newPageIndex) {		
		if (currentPageIndex == newPageIndex)				// Self to Self (probably a worakround callback)
			return;
		else if (currentPageIndex == textEditorIndex) {		// Text to XML
			if (textEditor.isDirty()) {
				setActivePage(currentPageIndex);		// Inhibit change for now.
				if (!pageChangePrompt(textEditor, xmlEditor))
		        	return;
				currentPageIndex = newPageIndex;
				setActivePage(currentPageIndex);		// Do change for now.
			}
		}
		else if (currentPageIndex  == xmlEditorIndex) {		// XML to Text
			if (xmlEditor.isDirty()) {
				setActivePage(currentPageIndex);		// Inhibit change for now.
				if (!pageChangePrompt(xmlEditor, textEditor))
		        	return;
				currentPageIndex = newPageIndex;
				setActivePage(currentPageIndex);		// Do change for now.
			}
		}
		if (currentPageIndex == -1)
			; 				// Going from init to display of first page
		else if (newPageIndex == textEditorIndex)
			textEditor.setInput(getTextEditorInput());
		else if (newPageIndex == xmlEditorIndex)
			xmlEditor.setInput(getXMLEditorInput());
		else if (newPageIndex == ecoreEditorIndex)
			xmlEditor.setInput(getEcoreEditorInput());
		else if (newPageIndex == emofEditorIndex)
			xmlEditor.setInput(getEMOFEditorInput());
		currentPageIndex = newPageIndex;
		super.pageChange(newPageIndex);
		setPartName(masterEditor.getTitle());
		IFileEditorInput textInput = (IFileEditorInput) textEditor.getEditorInput();
		refreshMarkers(textInput.getFile());
	}

	protected boolean pageChangePrompt(StatusTextEditor oldEditor, StatusTextEditor newEditor) {
		StringBuffer s = new StringBuffer();
		boolean canAbort = !(newEditor.getEditorInput() instanceof DummyFileEditorInput);
		s.append("'" + textEditor.getTitle() + "' has been modified.\n");
		s.append("\n  Proceed - save changes to '" + oldEditor.getTitle() + "'");
		if (!canAbort)
			s.append("\n                - start editing '" + newEditor.getTitle() + "'");
		else {
			s.append("\n                - overwrite and edit '" + newEditor.getTitle() + "'");
			s.append("\n\n  Abort - discard changes to '" + oldEditor.getTitle() + "'");
			s.append("\n            - revert to editing '" + newEditor.getTitle() + "'");
		}
		s.append("\n\n  Cancel - resume editing '" + oldEditor.getTitle() + "'");				
		String[] buttons = canAbort
			? new String[] { IDialogConstants.PROCEED_LABEL, IDialogConstants.ABORT_LABEL, IDialogConstants.CANCEL_LABEL }
			: new String[] { IDialogConstants.PROCEED_LABEL, IDialogConstants.CANCEL_LABEL };
		MessageDialog dialog = new MessageDialog(getSite().getShell(), "Save Resource", null,
		        s.toString(), MessageDialog.QUESTION, buttons, 0); // proceed is the default
		int buttonId = dialog.open();
        if (buttonId >= (canAbort ? 2 : 1))
        	return false;
        if (buttonId > 0)
        	oldEditor.doRevertToSaved();
        else {
        	oldEditor.doSave(null);
			masterEditor = oldEditor;
        }
		return true;
	}
	
	protected abstract Object refreshMarkers(IFile textFile);
}
