/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.ocl.cst.environment;

import java.io.IOException;
import java.util.Collection;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.ocl.cst.CSTNode;
import org.eclipse.qvt.declarative.ecore.utils.XMIUtils;
import org.eclipse.qvt.declarative.emof.EssentialOCL.util.EssentialOCLMappingMetaData;
import org.eclipse.qvt.declarative.registry.environment.AbstractModelResolver;

public class OCLTopLevelEnvironment extends OCLEnvironment
{
	protected AbstractModelResolver resolver = null;
	
	public OCLTopLevelEnvironment(EPackage.Registry reg, CSTNode cstNode) {
		super(reg);
		setCSTNode(cstNode);
	}

	public Resource createASTResource(Collection<? extends EObject> asts, URI uri) {
		Resource resource = getResourceSet().createResource(uri, getContentTypeIdentifier());
		if (resource != null) {
			if (asts != null)
				for (EObject ast : asts)
					if (ast != null)
						resource.getContents().add(ast);
			XMIUtils.assignIds(resource, "ast");
		}
		return resource;
	}

	protected String getContentTypeIdentifier() {
		return EssentialOCLMappingMetaData.INSTANCE.getEcoreContentTypeIdentifier();	// FIXME Use an MDT-OCL defined value once available
	}

	public ResourceSet getResourceSet() {
		return resolver.getResourceSet();
	}

	public void saveCST(CSTNode cstNode, URI uri) throws IOException {
		Resource resource = getResourceSet().createResource(uri);
		XMIUtils.assignIds(resource, "cst");
		resource.save(null);
	}
}
