/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.qvt.ui.preferences;

import org.eclipse.gmt.umlx.qvt.ui.QVTUIPlugin;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Class used to access preference values.
 */
public class QVTPreferences
{
	private static String representation = null;
	private static boolean generateEMOF = false;
	private static String[] coreEMOFExtensions = null;
	private static String[] coreEcoreExtensions = null;
	private static String[] coreTextExtensions = null;
	private static String[] km3EMOFExtensions = null;
	private static String[] km3EcoreExtensions = null;
	private static String[] km3TextExtensions = null;
	private static String[] oclEMOFExtensions = null;
	private static String[] oclEcoreExtensions = null;
	private static String[] oclTextExtensions = null;
	private static String[] operationalEMOFExtensions = null;
	private static String[] operationalEcoreExtensions = null;
	private static String[] operationalTextExtensions = null;
	private static String[] relationEMOFExtensions = null;
	private static String[] relationEcoreExtensions = null;
	private static String[] relationTextExtensions = null;
	
	
	public static void clean() {
		representation = null;
		coreEMOFExtensions = null;		
		coreEcoreExtensions = null;		
		coreTextExtensions = null;		
		km3EMOFExtensions = null;		
		km3EcoreExtensions = null;		
		km3TextExtensions = null;		
		oclEMOFExtensions = null;		
		oclEcoreExtensions = null;		
		oclTextExtensions = null;		
		operationalEMOFExtensions = null;		
		operationalEcoreExtensions = null;		
		operationalTextExtensions = null;		
		relationEMOFExtensions = null;		
		relationEcoreExtensions = null;		
		relationTextExtensions = null;		
	}
	
	public static boolean generateEMOF() {
		if (representation == null) {
			getRepresentation();
			generateEMOF = "emof".equals(representation);
		}
		return generateEMOF;
	}
	
	public static String getExtension(String[] preferences) {
		if ((preferences == null) || (preferences.length < 1))
			return null;
		return preferences[0];
	}
	
	public static String[] getExtensions(String preferenceName) {
		IPreferenceStore store = QVTUIPlugin.getDefault().getPreferenceStore();
		String preference = store.getString(preferenceName);
		if (preference == null)
			return null;
		return preference.split(",");
	}
	
	public static String[] getCoreEMOFExtensions() {
		if (coreEMOFExtensions == null)
			coreEMOFExtensions = getExtensions(QVTPreferenceConstants.P_EMOF_EXTENSIONS.CORE);
		return coreEMOFExtensions;
	}
	
	public static String[] getCoreEcoreExtensions() {
		if (coreEcoreExtensions == null)
			coreEcoreExtensions = getExtensions(QVTPreferenceConstants.P_ECORE_EXTENSIONS.CORE);
		return coreEcoreExtensions;
	}
	
	public static String getCoreTextExtension() {
		return getExtension(getCoreTextExtensions());
	}
	
	public static String[] getCoreTextExtensions() {
		if (coreTextExtensions == null)
			coreTextExtensions = getExtensions(QVTPreferenceConstants.P_TEXT_EXTENSIONS.CORE);
		return coreTextExtensions;
	}
	
//	public static String getCoreXMLExtension() {
//		return getExtension(getCoreXMLExtensions());
//	}
	
//	public static String[] getCoreXMLExtensions() {
//		if (coreXMLExtensions == null)
//			coreXMLExtensions = getExtensions(getXMLExtensionPreferenceNames().CORE);
//		return coreXMLExtensions;
//	}
	
	public static String getKM3EMOFExtension() {
		return getExtension(getKM3EMOFExtensions());
	}
	
	public static String[] getKM3EMOFExtensions() {
		if (km3EMOFExtensions == null)
			km3EMOFExtensions = getExtensions(QVTPreferenceConstants.P_EMOF_EXTENSIONS.KM3);
		return km3EMOFExtensions;
	}
	
	public static String getKM3EcoreExtension() {
		return getExtension(getKM3EcoreExtensions());
	}
	
	public static String[] getKM3EcoreExtensions() {
		if (km3EcoreExtensions == null)
			km3EcoreExtensions = getExtensions(QVTPreferenceConstants.P_ECORE_EXTENSIONS.KM3);
		return km3EcoreExtensions;
	}

	public static String getKM3TextExtension() {
		return getExtension(getKM3TextExtensions());
	}
	
	public static String[] getKM3TextExtensions() {
		if (km3TextExtensions == null)
			km3TextExtensions = getExtensions(QVTPreferenceConstants.P_TEXT_EXTENSIONS.KM3);
		return km3TextExtensions;
	}
	
	public static String getOCLEMOFExtension() {
		return getExtension(getOCLEMOFExtensions());
	}
	
	public static String[] getOCLEMOFExtensions() {
		if (oclEMOFExtensions == null)
			oclEMOFExtensions = getExtensions(QVTPreferenceConstants.P_EMOF_EXTENSIONS.OCL);
		return oclEMOFExtensions;
	}
	
	public static String getOCLEcoreExtension() {
		return getExtension(getOCLEcoreExtensions());
	}
	
	public static String[] getOCLEcoreExtensions() {
		if (oclEcoreExtensions == null)
			oclEcoreExtensions = getExtensions(QVTPreferenceConstants.P_ECORE_EXTENSIONS.OCL);
		return oclEcoreExtensions;
	}

	public static String getOCLTextExtension() {
		return getExtension(getOCLTextExtensions());
	}
	
	public static String[] getOCLTextExtensions() {
		if (oclTextExtensions == null)
			oclTextExtensions = getExtensions(QVTPreferenceConstants.P_TEXT_EXTENSIONS.OCL);
		return oclTextExtensions;
	}
	
	public static String getOperationalEMOFExtension() {
		return getExtension(getOperationalEMOFExtensions());
	}
	
	public static String[] getOperationalEMOFExtensions() {
		if (operationalEMOFExtensions == null)
			operationalEMOFExtensions = getExtensions(QVTPreferenceConstants.P_EMOF_EXTENSIONS.OPERATIONAL);
		return operationalEMOFExtensions;
	}
	
	public static String getOperationalEcoreExtension() {
		return getExtension(getOperationalEcoreExtensions());
	}
	
	public static String[] getOperationalEcoreExtensions() {
		if (operationalEcoreExtensions == null)
			operationalEcoreExtensions = getExtensions(QVTPreferenceConstants.P_ECORE_EXTENSIONS.OPERATIONAL);
		return operationalEcoreExtensions;
	}

	public static String getOperationalTextExtension() {
		return getExtension(getOperationalTextExtensions());
	}
	
	public static String[] getOperationalTextExtensions() {
		if (operationalTextExtensions == null)
			operationalTextExtensions = getExtensions(QVTPreferenceConstants.P_TEXT_EXTENSIONS.OPERATIONAL);
		return operationalTextExtensions;
	}
	
	public static String getRelationEMOFExtension() {
		return getExtension(getRelationEMOFExtensions());
	}
	
	public static String[] getRelationEMOFExtensions() {
		if (relationEMOFExtensions == null)
			relationEMOFExtensions = getExtensions(QVTPreferenceConstants.P_EMOF_EXTENSIONS.RELATION);
		return relationEMOFExtensions;
	}
	
	public static String getRelationEcoreExtension() {
		return getExtension(getRelationEcoreExtensions());
	}
	
	public static String[] getRelationEcoreExtensions() {
		if (relationEcoreExtensions == null)
			relationEcoreExtensions = getExtensions(QVTPreferenceConstants.P_ECORE_EXTENSIONS.RELATION);
		return relationEcoreExtensions;
	}
	
	public static String getRelationTextExtension() {
		return getExtension(getRelationTextExtensions());
	}
	
	public static String[] getRelationTextExtensions() {
		if (relationTextExtensions == null)
			relationTextExtensions = getExtensions(QVTPreferenceConstants.P_TEXT_EXTENSIONS.RELATION);
		return relationTextExtensions;
	}
	
	public static String getRepresentation() {
		if (representation == null) {
			IPreferenceStore store = QVTUIPlugin.getDefault().getPreferenceStore();
			representation = store.getString(QVTPreferenceConstants.P_QVT_REPRESENTATION);
		}
		return representation;
	}

	public static boolean hasExtension(String fileExtension, String[] extensions) {
		if (fileExtension == null)
			return false;
		if (extensions == null)
			return false;
		for (String extension : extensions)
			if (fileExtension.compareToIgnoreCase(extension) == 0)
				return true;
		return false;
	}

	public static boolean hasCoreTextExtension(String fileExtension) {
		return hasExtension(fileExtension, getCoreTextExtensions());
	}

	public static boolean hasKM3TextExtension(String fileExtension) {
		return hasExtension(fileExtension, getKM3TextExtensions());
	}

	public static boolean hasOCLTextExtension(String fileExtension) {
		return hasExtension(fileExtension, getOCLTextExtensions());
	}

	public static boolean hasOperationalTextExtension(String fileExtension) {
		return hasExtension(fileExtension, getOperationalTextExtensions());
	}

	public static boolean hasRelationTextExtension(String fileExtension) {
		return hasExtension(fileExtension, getRelationTextExtensions());
	}
}
