/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.qvt.declarative.registry.model;

import org.eclipse.emf.common.util.URI;
import org.eclipse.qvt.declarative.registry.environment.AbstractFileHandle;
import org.eclipse.qvt.declarative.registry.util.ClassUtils;

/**
 * A Registration maintains the binding between an accessor and the URI by which
 * the model may be loaded. Registrations are maintained in the hierarchy of
 * ProjectRegistry containing FileHandleRegistry containing AccessorRegistry containing Registration.
 */
public class Registration<A extends Accessor> implements Comparable<Registration<?>>
{
	private final AccessorRegistry<A> parent;
	private final A accessor;
	private final URI uri;
	private final ModelSerialization modelSerialization;
	
	public Registration(AccessorRegistry<A> parent, A accessor, URI uri, ModelSerialization modelSerialization) {
		this.parent = parent;
		this.accessor = accessor;
		this.uri = uri;
		this.modelSerialization = modelSerialization;
	}

	public int compareTo(Registration<?> registration) {
		int comparison = getFileHandleRegistry().compareTo(registration.getFileHandleRegistry());
		if (comparison != 0)
			return comparison;
		return getAccessorName().compareTo(registration.getAccessorName());
	}

	@Override public boolean equals(Object obj) {
		if (!(obj instanceof Registration))
			return false;
		if (!getAccessorName().equals(((Registration<?>)obj).getAccessorName()))
			return false;
		if (!getSerializationName().equals(((Registration<?>)obj).getSerializationName()))
			return false;
		if (!getURI().equals(((Registration<?>)obj).getURI()))
			return false;
		return true;
	}
	
	public A getAccessor() { return accessor; }
	public Class<A> getAccessorClass() { return ClassUtils.getClass(accessor); }
	public String getAccessorName() { return getAccessor().getName(); }
	public AccessorRegistry<A> getAccessorRegistry() { return parent; }
	public AbstractFileHandle getFileHandle() { return parent.getFileHandle(); }
	public FileHandleRegistry getFileHandleRegistry() { return parent.getFileHandleRegistry(); }
	public String getModelKindName() { return modelSerialization.getName(); }
	public ModelSerialization getSerialization() { return modelSerialization; }
	public String getSerializationName() { return modelSerialization.getName(); }
	public URI getURI() { return uri; }
	public String getURIString() { return getURI().toString(); }

	@Override public String toString() {
		return getAccessorName() + " -> " + getURIString();
	}

}
