package org.eclipse.gmt.umlx.eqvtc.tests.editor;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gmt.umlx.editor.ICreationFactory;
import org.eclipse.gmt.umlx.editor.paged.TestProgressMonitor;
import org.eclipse.gmt.umlx.editor.text.TextPageManager;
import org.eclipse.gmt.umlx.editor.xml.EMOFXMLPageFactory;
import org.eclipse.gmt.umlx.editor.xml.EMOFXMLPageManager;
import org.eclipse.gmt.umlx.editor.xml.EcoreXMLPageFactory;
import org.eclipse.gmt.umlx.editor.xml.EcoreXMLPageManager;
import org.eclipse.gmt.umlx.eqvt.tests.EditorTestCase;
import org.eclipse.gmt.umlx.eqvtc.editor.QVTcEditor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.text.AbstractDocument;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.IDocumentProvider;

public class QVTcEditingTests2 extends EditorTestCase
{
	@Override
	protected String getEditorId() {
		return "org.eclipse.gmt.umlx.eqvtc.editor.QVTcEditor";
	}

	public void testExitEcoreThenEMOFthenText() throws CoreException, ExecutionException, BadLocationException {		
		final String registryContents =
			"<?xml version=\"1.0\" encoding=\"ASCII\"?>\n" +
			"<mreg:ModelRegistrySettings xmi:version=\"2.0\" xmlns:xmi=\"http://www.omg.org/XMI\" xmlns:mreg=\"http://www.eclipse.org/gmt/umlx/2007/ModelRegistry\">\n" +
			"  <resource name=\"" + getName() + ".qvtc\">\n" +
			"    <entry accessor=\"tuml2rdbms\" kind=\"Model Name\" uri=\"TumlRdbms.ecore\"/>\n" +
			"  </resource>\n" +
			"</mreg:ModelRegistrySettings>\n";
		IFolder folder = createFolder(".settings");
		createFile(folder, "org.eclipse.gmt.umlx.registry.modelregistry", registryContents);
		final String metaModelContents =
			"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
			"<ecore:EPackage xmi:version=\"2.0\"\n" +
			"    xmlns:xmi=\"http://www.omg.org/XMI\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" +
			"    xmlns:ecore=\"http://www.eclipse.org/emf/2002/Ecore\" name=\"tuml2rdbms\"\n" +
			"    nsPrefix=\"\">\n" +
			"  <eClassifiers xsi:type=\"ecore:EClass\" name=\"TClassToTable\"/>\n" +
			"</ecore:EPackage>\n";
		createFile("TumlRdbms.ecore", metaModelContents);
		final String contents =
			"transformation umlRdbms {\n" +
			"	middle imports tuml2rdbms;\n" +
			"}\n" +
			"\n" +
			"map ClassToTable in umlRdbms\n" +
			"{\n" +
			"	where () {}\n" +
			"}\n";
		final String pageTitle = "QVT core Text";
		final String ecorePageTitle = "Ecore XMI";
		final String emofPageTitle = "EMOF XMI";
		String fileName = getName() + ".qvtc";
		IFile file = createFile(fileName, contents);
		IFileEditorInput editorInput = new FileEditorInput(file);
		QVTcEditor editor = (QVTcEditor) workbenchPage.openEditor(editorInput, getEditorId());
		try {
			final Display display = editor.getDisplay();
			ICreationFactory creationFactory = editor.getCreationFactory();		
			String markerId = creationFactory.getProblemMarkerId();	
	
			assertEquals("Page managers", 4, editor.getPageManagers().size());
			TextPageManager textPageManager = (TextPageManager) editor.getActivePageManager();
			TextEditor textEditor = textPageManager.getEditor();
			TextEditor ecoreEditor = (TextEditor) editor.getEditor(EcoreXMLPageFactory.PAGE_TITLE);
			int ecorePageIndex = editor.getPageIndex(ecoreEditor);
			assertTrue("Valid Ecore Page Index", (0 <= ecorePageIndex) && (ecorePageIndex < 4));
			TextEditor emofEditor = (TextEditor) editor.getEditor(EMOFXMLPageFactory.PAGE_TITLE);
			int emofPageIndex = editor.getPageIndex(emofEditor);
			assertTrue("Valid EMOF Page Index", (0 <= emofPageIndex) && (emofPageIndex < 4));
			
			IDocumentProvider documentProvider = textEditor.getDocumentProvider();
			AbstractDocument document = (AbstractDocument) documentProvider.getDocument(editorInput);
			IMarker[] markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
					
			assertEquals("Original contents", contents, document.get());
			assertEquals("Original isDirty", false, editor.isDirty());
			assertEquals("Original isDirtyPage", false, editor.isDirtyPage());
			assertEquals("Original editor title", fileName, editor.getTitle());
			assertEquals("Original Text page title", pageTitle, editor.getPageTitle(textEditor));
			assertEquals("Original markers", 0, markers.length);
	
			editor.setActivePage(ecorePageIndex);
	//		editor.getOperationHistory().undo(editor.getUndoContext(), monitor, null);
			runAsyncMessages(display, "Page Change To Ecore");
	
			assertEquals("Original Ecore isDirty", false, editor.isDirty());
			assertEquals("Original Ecore isDirtyPage", false, editor.isDirtyPage());
			assertEquals("Original Ecore editor title", fileName, editor.getTitle());
			assertEquals("Original Ecore page title", pageTitle, editor.getPageTitle(textEditor));
			assertEquals("Original Ecore markers", 0, markers.length);
	
			EcoreXMLPageManager ecorePageManager = (EcoreXMLPageManager) editor.getActivePageManager();		
			IDocumentProvider ecoreDocumentProvider = ecoreEditor.getDocumentProvider();
			AbstractDocument ecoreDocument = (AbstractDocument) ecoreDocumentProvider.getDocument(ecoreEditor.getEditorInput());
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
					
			String ecoreContents = ecoreDocument.get();
			int ecoreEditAt = ecoreContents.indexOf("ClassToTable");
			assertTrue("Valid Ecore Context", ecoreEditAt >= 0);
			ecoreDocument.replace(ecoreEditAt, 0, "Modified");
			runAsyncMessages(display, "Ecore Page Typing");
			String ecoreContents2 = ecoreDocument.get();
	
			assertEquals("Ecore Edited isDirty", true, editor.isDirty());
			assertEquals("Ecore Edited isDirtyPage", true, editor.isDirtyPage());
			String xx = editor.getTitle();
//			assertEquals("Ecore Edited editor title", "*" + fileName, editor.getTitle());
			assertEquals("Ecore Edited page title", "*" + ecorePageTitle, editor.getPageTitle(ecoreEditor));
			assertEquals("Ecore Edited Text page title", pageTitle, editor.getPageTitle(textEditor));
			textPageManager.refresh();
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
			assertEquals("Ecore Edited markers", 0, markers.length);
	
			final TestProgressMonitor testMonitor = new TestProgressMonitor();
			display.asyncExec(new Runnable()
			{
				public void run() {
					testMonitor.actions.get(IDialogConstants.OK_LABEL).run();
				}
			});
			editor.doSave(testMonitor);
	//		editor.getOperationHistory().undo(editor.getUndoContext(), monitor, null);
			runAsyncMessages(display, "Ecore Save");
	
			assertEquals("Saved Ecore isDirty", false, editor.isDirty());
			assertEquals("Saved Ecore isDirtyPage", false, editor.isDirtyPage());
//			assertEquals("Saved Ecore editor title", fileName, editor.getTitle());
			assertEquals("Saved Ecore page title", ecorePageTitle, editor.getPageTitle(ecorePageManager.getEditor()));
			assertEquals("Saved Ecore Text page title", pageTitle, editor.getPageTitle(textEditor));
			textPageManager.refresh();
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
			assertEquals("Saved Ecore markers", 0, markers.length);
	
			editor.setActivePage(emofPageIndex);
	//		editor.getOperationHistory().undo(editor.getUndoContext(), monitor, null);
			runAsyncMessages(display, "Page Change To EMOF");
	
			EMOFXMLPageManager emofPageManager = (EMOFXMLPageManager) editor.getActivePageManager();		
			assertEquals("Original EMOF isDirty", false, editor.isDirty());
			assertEquals("Original EMOF isDirtyPage", false, editor.isDirtyPage());
			assertEquals("Original EMOF page title", emofPageTitle, editor.getPageTitle(emofPageManager.getEditor()));
			assertEquals("Original EMOF markers", 0, markers.length);
	
			IDocumentProvider emofDocumentProvider = emofEditor.getDocumentProvider();
			AbstractDocument emofDocument = (AbstractDocument) emofDocumentProvider.getDocument(emofEditor.getEditorInput());
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
					
			String emofContents = emofDocument.get();
			int emofEditAt = emofContents.indexOf("ClassToTable");
			assertTrue("Valid EMOF Context", emofEditAt >= 0);
			emofDocument.replace(emofEditAt, 0, "Again");
			runAsyncMessages(display, "EMOF Page Typing");
	
			assertEquals("EMOF Edited isDirty", true, editor.isDirty());
			assertEquals("EMOF Edited isDirtyPage", true, editor.isDirtyPage());
			assertEquals("EMOF Edited page title", "*" + emofPageTitle, editor.getPageTitle(textEditor));
			textPageManager.refresh();
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
			assertEquals("EMOF Edited markers", 0, markers.length);
			
	/*		assertEquals("Edited contents", contents1 + contents2 + contents3, document.get());
			assertEquals("Edited isDirty", true, editor.isDirty());
			assertEquals("Edited isDirtyPage", true, editor.isDirtyPage());
			assertEquals("Edited page title", "*" + pageTitle, editor.getPageTitle(textEditor));
			textPageManager.refresh();
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
			assertEquals("Edited markers", 1, markers.length);
			assertEquals("Edited marker", "Parser: merge tokens \"trans formation\" to form \"transformation\"", markers[0].getAttribute(IMarker.MESSAGE, ""));
			
			editor.getOperationHistory().undo(editor.getUndoContext(), monitor, null);
			runAsyncMessages(editor.getDisplay(), "Undo");
	
			assertEquals("Restored contents", contents, document.get());
			assertEquals("Restored isDirty", false, editor.isDirty());
			assertEquals("Restored isDirtyPage", false, editor.isDirtyPage());
			assertEquals("Restored page title", pageTitle, editor.getPageTitle(textEditor));
			textPageManager.refresh();
			markers = file.findMarkers(markerId, true, IResource.DEPTH_INFINITE);
			assertEquals("Restored markers", 0, markers.length);
			assertTrue("Close Editor", workbenchPage.closeEditor(editor, true)); */
		}
		finally {
			workbenchPage.closeEditor(editor, false);
		}
	}
}
