/*******************************************************************************
 * Copyright (c) 2008 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.eqvtr.tests;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.gmt.umlx.eqvt.tests.AbstractConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.EMOFConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.EssentialOCLConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EcoreDifference;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EquivalenceHelper;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EquivalenceMap;
import org.eclipse.qvt.declarative.ecore.mappings.IMappingMetaDataRegistry;
import org.eclipse.qvt.declarative.ecore.mappings.MappingConfigurationException;
import org.eclipse.qvt.declarative.emof.QVTRelation.QVTRelationPackage;
import org.eclipse.qvt.declarative.emof.QVTRelation.util.QVTRelationMappingMetaData;

public class QVTRelationConsistencyTest extends AbstractConsistencyTest
{	
	public static void expectedEcoreDifferences(EquivalenceHelper helper, Set<EcoreDifference> differences) {
		changeOfURI(differences, helper, org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.eINSTANCE);
		changeOfPrefix(differences, helper, org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.eINSTANCE);
		//
		//	Ecore added value
		//
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.Literals.RELATION, EcorePackage.Literals.ECLASS__EOPERATIONS);
		missingOperation(differences, helper, getOperation(org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.Literals.RELATIONAL_TRANSFORMATION, "getRelation"));
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.Literals.RELATIONAL_TRANSFORMATION, EcorePackage.Literals.ECLASS__EOPERATIONS);		
		missingOperation(differences, helper, getOperation(org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.Literals.RELATION, "getRelationalTransformation"));
		missingConstraint(differences, helper, org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.Literals.RELATION_CALL_EXP);
	}

	@Override
	protected Set<EcoreDifference> getExpectedEcore2EmofDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		QVTRelationConsistencyTest.expectedEcoreDifferences(helper, differences);		
		helper.setLogStream(null);
		EssentialOCLConsistencyTest.expectedEcoreDifferences(helper, differences);
		EMOFConsistencyTest.expectedEcore2RoseDifferences(helper, differences, rightPackage.eResource().getResourceSet());		
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgEcoreDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfURI(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfPrefix(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfName(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_CALL_EXP__ARGUMENT);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_CALL_EXP__REFERRED_RELATION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__PATTERN);
		changeOfContainment(differences, helper, QVTRelationPackage.Literals.RELATION__OPERATIONAL_IMPL);
		changeOfTransient(differences, helper, QVTRelationPackage.Literals.RELATION_IMPLEMENTATION__RELATION);
		changeOfOrdered(differences, helper, QVTRelationPackage.Literals.KEY__PART);
		changeOfOrdered(differences, helper, QVTRelationPackage.Literals.RELATION__OPERATIONAL_IMPL);
		changeOfOrdered(differences, helper, QVTRelationPackage.Literals.RELATION__VARIABLE);
		changeOfOrdered(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__DEFAULT_ASSIGNMENT);
		changeOfOrdered(differences, helper, QVTRelationPackage.Literals.RELATIONAL_TRANSFORMATION__OWNED_KEY);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION__WHEN);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION__WHERE);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__DEFAULT_ASSIGNMENT);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN_ASSIGNMENT__VALUE_EXP);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN_ASSIGNMENT__VARIABLE);
		helper.setLogStream(null);
		EMOFConsistencyTest.expectedEmofDifferences(helper, differences);		
		EssentialOCLConsistencyTest.expectedEmofDifferences(helper, differences);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgEmofDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		missingURI(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfName(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfContainment(differences, helper, QVTRelationPackage.Literals.RELATION__OPERATIONAL_IMPL);
		changeOfTransient(differences, helper, QVTRelationPackage.Literals.KEY__TRANSFORMATION);
		changeOfTransient(differences, helper, QVTRelationPackage.Literals.RELATION_IMPLEMENTATION__RELATION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.DOMAIN_PATTERN__TEMPLATE_EXPRESSION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.KEY__TRANSFORMATION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION__IS_TOP_LEVEL);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION__OPERATIONAL_IMPL);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION__VARIABLE);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION__WHEN);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION__WHERE);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_CALL_EXP__ARGUMENT);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__DEFAULT_ASSIGNMENT);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_IMPLEMENTATION__RELATION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATIONAL_TRANSFORMATION__OWNED_KEY);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION__WHEN);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION__WHERE);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__DEFAULT_ASSIGNMENT);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN_ASSIGNMENT__VALUE_EXP);
		missingOpposite(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN_ASSIGNMENT__VARIABLE);
		helper.setLogStream(null);
		missingPrefix(differences, helper, QVTRelationPackage.eINSTANCE);
		missingURI(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		missingPrefix(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		changeOfName(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE, EcorePackage.Literals.EPACKAGE__ECLASSIFIERS);
		missingURI(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		missingPrefix(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		changeOfName(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE, EcorePackage.Literals.EPACKAGE__ECLASSIFIERS);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgRoseDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfURI(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfPrefix(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfName(differences, helper, QVTRelationPackage.eINSTANCE);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_CALL_EXP__ARGUMENT);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_CALL_EXP__REFERRED_RELATION);
		changeOfLower(differences, helper, QVTRelationPackage.Literals.RELATION_DOMAIN__PATTERN);
		helper.setLogStream(null);
		EMOFConsistencyTest.expectedEmofDifferences(helper, differences);		
		EssentialOCLConsistencyTest.expectedEmofDifferences(helper, differences);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2RoseDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfPrefix(differences, helper, QVTRelationPackage.eINSTANCE);		
		return differences;
	}

	@Override
	protected IMappingMetaDataRegistry getMappingMetaDataRegistry() {
		IMappingMetaDataRegistry mappingMetaDataRegistry = super.getMappingMetaDataRegistry();
		QVTRelationMappingMetaData.INSTANCE.getFactory(mappingMetaDataRegistry);
		return mappingMetaDataRegistry;
	}

	@Override
	public String getPackageName() {
		return "QVTRelation";
	}

	@Override
	protected void installEcoreMappings(EquivalenceMap comparator, ResourceSet resourceSet, boolean isOMG) {
		super.installEcoreMappings(comparator, resourceSet, isOMG);
		comparator.putEquivalence(org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.eINSTANCE, getPackage(resourceSet, getPackageName(isOMG)));
	}

	@Override
	protected void installEmofMappings(EquivalenceMap comparator, ResourceSet resourceSet, boolean isOMG) {
		super.installEmofMappings(comparator, resourceSet, isOMG);
		comparator.putEquivalence(QVTRelationPackage.eINSTANCE, getPackage(resourceSet, getPackageName(isOMG)));
	}

	@Override
	protected void installEmof2EmofMappings(EquivalenceMap comparator, ResourceSet resourceSet) {
		super.installEmof2EmofMappings(comparator, resourceSet);
		comparator.putEquivalence(QVTRelationPackage.eINSTANCE, getPackage(resourceSet, getPackageName(true)));
	}
	
	public void testQVTRelationConsistency() throws IOException, MappingConfigurationException {
		Map<InconsistencyKey, InconsistencyValue> ecoreInconsistencies = new HashMap<InconsistencyKey, InconsistencyValue>();
		Map<InconsistencyKey, InconsistencyValue> adaptingInconsistencies = new HashMap<InconsistencyKey, InconsistencyValue>();
		getExpectedDerivedPackageInconsistencies(ecoreInconsistencies, adaptingInconsistencies);
		ecore2EmofAdapterConsistencyTest(ecoreInconsistencies, adaptingInconsistencies);
		emof2OmgRoseConsistencyTest(QVTRelationPackage.eNS_URI, "qvtrelation.ecore");
		emof2OmgEcoreConsistencyTest(QVTRelationPackage.eNS_URI, "qvtrelation.ecore");
		emof2OmgEmofConsistencyTest(QVTRelationPackage.eNS_URI);
		emof2RoseConsistencyTest(QVTRelationPackage.eNS_URI, "QVTRelation.ecore");
		ecore2EmofConsistencyTest(org.eclipse.qvt.declarative.ecore.QVTRelation.QVTRelationPackage.eNS_URI,
				QVTRelationPackage.eINSTANCE,
				org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE,
				org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
	}
}
