/*******************************************************************************
 * Copyright (c) 2008 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.eqvtr.tests;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.gmt.umlx.eqvt.tests.AbstractConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.EMOFConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.EssentialOCLConsistencyTest;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EcoreDifference;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EquivalenceHelper;
import org.eclipse.gmt.umlx.eqvt.tests.tools.EquivalenceMap;
import org.eclipse.qvt.declarative.ecore.mappings.IMappingMetaDataRegistry;
import org.eclipse.qvt.declarative.ecore.mappings.MappingConfigurationException;
import org.eclipse.qvt.declarative.emof.QVTTemplate.QVTTemplatePackage;
import org.eclipse.qvt.declarative.emof.QVTTemplate.util.QVTTemplateMappingMetaData;

public class QVTTemplateConsistencyTest extends AbstractConsistencyTest
{	
	public static void expectedEcoreDifferences(EquivalenceHelper helper, Set<EcoreDifference> differences) {
		changeOfURI(differences, helper, org.eclipse.qvt.declarative.ecore.QVTTemplate.QVTTemplatePackage.eINSTANCE);
		changeOfPrefix(differences, helper, org.eclipse.qvt.declarative.ecore.QVTTemplate.QVTTemplatePackage.eINSTANCE);
		//
		//	Ecore added value
		//
		missingConstraint(differences, helper, org.eclipse.qvt.declarative.ecore.QVTTemplate.QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP);
	}

	@Override
	protected Set<EcoreDifference> getExpectedEcore2EmofDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		QVTTemplateConsistencyTest.expectedEcoreDifferences(helper, differences);
		helper.setLogStream(null);
		EssentialOCLConsistencyTest.expectedEcoreDifferences(helper, differences);
		EMOFConsistencyTest.expectedEcore2RoseDifferences(helper, differences, rightPackage.eResource().getResourceSet());		
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgEcoreDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfURI(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfPrefix(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfName(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfObject(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST, EcorePackage.Literals.ETYPED_ELEMENT__ETYPE);
		changeOfSize(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP, EcorePackage.Literals.ECLASS__ESTRUCTURAL_FEATURES);
		extraObject(differences, helper, getFeature(getClass(rightPackage, "CollectionTemplateExp"), "kind"));
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REFERRED_COLLECTION_TYPE);
		changeOfOrdered(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		changeOfOrdered(differences, helper, QVTTemplatePackage.Literals.OBJECT_TEMPLATE_EXP__PART);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.PROPERTY_TEMPLATE_ITEM__REFERRED_PROPERTY);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.PROPERTY_TEMPLATE_ITEM__VALUE);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.TEMPLATE_EXP__WHERE);
		helper.setLogStream(null);
		EMOFConsistencyTest.expectedEmofDifferences(helper, differences);		
		EssentialOCLConsistencyTest.expectedEmofDifferences(helper, differences);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgEmofDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		missingURI(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfName(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfObject(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST, EcorePackage.Literals.ETYPED_ELEMENT__ETYPE);
		changeOfTransient(differences, helper, QVTTemplatePackage.Literals.PROPERTY_TEMPLATE_ITEM__OBJ_CONTAINER);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.OBJECT_TEMPLATE_EXP__PART);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.TEMPLATE_EXP__BINDS_TO);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.TEMPLATE_EXP__WHERE);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.PROPERTY_TEMPLATE_ITEM__REFERRED_PROPERTY);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.PROPERTY_TEMPLATE_ITEM__VALUE);
		missingOpposite(differences, helper, QVTTemplatePackage.Literals.TEMPLATE_EXP__WHERE);
		helper.setLogStream(null);
		missingPrefix(differences, helper, QVTTemplatePackage.eINSTANCE);
		missingURI(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		missingPrefix(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		changeOfName(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE);
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE, EcorePackage.Literals.EPACKAGE__ECLASSIFIERS);
		missingURI(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		missingPrefix(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		changeOfName(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
		changeOfSize(differences, helper, org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE, EcorePackage.Literals.EPACKAGE__ECLASSIFIERS);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2OmgRoseDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfURI(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfPrefix(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfName(differences, helper, QVTTemplatePackage.eINSTANCE);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__MEMBER);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REST);
		changeOfLower(differences, helper, QVTTemplatePackage.Literals.COLLECTION_TEMPLATE_EXP__REFERRED_COLLECTION_TYPE);
		helper.setLogStream(null);
		EMOFConsistencyTest.expectedEmofDifferences(helper, differences);		
		EssentialOCLConsistencyTest.expectedEmofDifferences(helper, differences);
		return differences;
	}

	@Override
	protected Set<EcoreDifference> getExpectedEmof2RoseDifferences(EquivalenceHelper helper, EPackage rightPackage) {
		Set<EcoreDifference> differences = new HashSet<EcoreDifference>();
		changeOfPrefix(differences, helper, QVTTemplatePackage.eINSTANCE);			
		return differences;
	}

	@Override
	protected IMappingMetaDataRegistry getMappingMetaDataRegistry() {
		IMappingMetaDataRegistry mappingMetaDataRegistry = super.getMappingMetaDataRegistry();
		QVTTemplateMappingMetaData.INSTANCE.getFactory(mappingMetaDataRegistry);
		return mappingMetaDataRegistry;
	}

	@Override
	public String getPackageName() {
		return "QVTTemplate";
	}

	@Override
	protected void installEcoreMappings(EquivalenceMap comparator, ResourceSet resourceSet, boolean isOMG) {
		super.installEcoreMappings(comparator, resourceSet, isOMG);
		comparator.putEquivalence(org.eclipse.qvt.declarative.ecore.QVTTemplate.QVTTemplatePackage.eINSTANCE, getPackage(resourceSet, getPackageName(isOMG)));
	}

	@Override
	protected void installEmofMappings(EquivalenceMap comparator, ResourceSet resourceSet, boolean isOMG) {
		super.installEmofMappings(comparator, resourceSet, isOMG);
		comparator.putEquivalence(QVTTemplatePackage.eINSTANCE, getPackage(resourceSet, getPackageName(isOMG)));
	}

	@Override
	protected void installEmof2EmofMappings(EquivalenceMap comparator, ResourceSet resourceSet) {
		super.installEmof2EmofMappings(comparator, resourceSet);
		comparator.putEquivalence(QVTTemplatePackage.eINSTANCE, getPackage(resourceSet, getPackageName(true)));
	}
	
	public void testQVTTemplateConsistency() throws IOException, MappingConfigurationException {
		Map<InconsistencyKey, InconsistencyValue> ecoreInconsistencies = new HashMap<InconsistencyKey, InconsistencyValue>();
		Map<InconsistencyKey, InconsistencyValue> adaptingInconsistencies = new HashMap<InconsistencyKey, InconsistencyValue>();
		getExpectedDerivedPackageInconsistencies(ecoreInconsistencies, adaptingInconsistencies);
		ecore2EmofAdapterConsistencyTest(ecoreInconsistencies, adaptingInconsistencies);
		emof2OmgRoseConsistencyTest(QVTTemplatePackage.eNS_URI, "qvttemplate.ecore");
		emof2OmgEcoreConsistencyTest(QVTTemplatePackage.eNS_URI, "qvttemplate.ecore");
		emof2OmgEmofConsistencyTest(QVTTemplatePackage.eNS_URI);
		emof2RoseConsistencyTest(QVTTemplatePackage.eNS_URI, "QVTTemplate.ecore");
		ecore2EmofConsistencyTest(org.eclipse.qvt.declarative.ecore.QVTTemplate.QVTTemplatePackage.eNS_URI,
				QVTTemplatePackage.eINSTANCE,
				org.eclipse.qvt.declarative.emof.EssentialOCL.EssentialOCLPackage.eINSTANCE,
				org.eclipse.qvt.declarative.emof.EMOF.EMOFPackage.eINSTANCE);
	}
}
