/*******************************************************************************
 * Copyright (c) 2007 E.D.Willink and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     E.D.Willink - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.umlx.eqvtr.tests.parser;

import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmt.umlx.cst.environment.IFileEnvironment;
import org.eclipse.gmt.umlx.eqvt.tests.parser.AbstractParseTestCase;
import org.eclipse.gmt.umlx.eqvtr.cst.environment.QVTrTopLevelEnvironment;
import org.eclipse.gmt.umlx.eqvtr.cst.unparser.EqvtRelationUnparser;
import org.eclipse.qvt.declarative.ecore.mappings.IMappingMetaDataRegistry;
import org.eclipse.qvt.declarative.emof.QVTRelation.util.QVTRelationMappingMetaData;
import org.eclipse.qvt.declarative.registry.standalone.FileHandle;

public abstract class AbstractQVTrTestCase extends AbstractParseTestCase
{
	@Override protected IFileEnvironment createEnvironment(String fileName) throws IOException, CoreException {
		FileHandle fileHandle = getFileHandle(fileName);
		QVTrTopLevelEnvironment environment = new QVTrTopLevelEnvironment(fileHandle, resourceSet)
		{
			@Override
			protected void initializePackageNs(EPackage ePackage) {
				ePackage.setNsPrefix(ePackage.getName());
				StringBuilder s = new StringBuilder();
				s.append("http://");
				s.append(file.getProjectHandle().getName());
				s.append("/");
				s.append(file.getProjectRelativeName().replace(".unparsed", ""));
				s.append("/");
				initializePackageNs(s, ePackage);
				ePackage.setNsURI(s.toString());
			}			
		};
		return environment;
	}
	
	@Override protected EqvtRelationUnparser createUnparser(Resource referenceResource) {
		return new EqvtRelationUnparser(referenceResource);
	}

	@Override
	protected IMappingMetaDataRegistry getMappingMetaDataRegistry() {
		IMappingMetaDataRegistry mappingMetaDataRegistry = super.getMappingMetaDataRegistry();
		QVTRelationMappingMetaData.INSTANCE.getFactory(mappingMetaDataRegistry);
		return mappingMetaDataRegistry;
	}
	
	@Override protected String getTextExtension() {
		return "qvtr";
	}

	@Override protected String getXMLExtension(boolean generateEMOF) {
		return (generateEMOF ? "qvtrelation" :  "eqvtrelation");
	}
}
